package edu.ucf.cs.dml.simnets.measureme.analysis;

//command line parameters:-Djava.security.policy=c:\jini1_2\policy\policy.all -Doutrigger.spacename=JavaSpaces -Dcom.sun.jini.lookup.groups=public -DrunOption=2


import edu.ucf.cs.dml.simnets.measureme.MeasureMeGlobal;
import edu.ucf.cs.dml.simnets.measureme.server.*;
import edu.ucf.cs.dml.simnets.measureme.client.*;
import edu.ucf.cs.dml.simnets.measureme.dbase.*;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.TransactionManager;
import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;

import java.awt.*;
import java.util.*;
import javax.swing.UIManager;


public class NewAnalysisStation implements LeaseListener, ClientEntersStationListener, ClientLeavesStationListener, AnalysisServiceInterface {
  boolean packFrame=false;
  protected StatusDialog statusDialog = new StatusDialog();
  protected JavaSpace javaSpace=null;
  protected JavaSpacesStatus jsStatus=null;   //

  protected TransactionManager mgr=null;      //helps maintain state info across system
  protected AutomatedLeaseManager alm=null;   //maintains leases for transactions
  protected Transaction currentTransaction=null;
  protected Lease currentLease=null;
  protected ExhibitorWrapper theExhibitorWrapper=null;   //the current client in the station
  protected DatabaseQueryEngine dbQE=null; //performs queries // CEH 9/23/2002
  protected NewAnalysisStationFrame frame=null;
  protected Hashtable countryCodes=new Hashtable();
  protected Hashtable countyCodes=new Hashtable();
  protected ClientAdministration clAdmin=null;


  public NewAnalysisStation() throws java.sql.SQLException,java.lang.ClassNotFoundException
  {
    this.dbQE=new DatabaseQueryEngine("org.postgresql.Driver","jdbc:postgresql://"+MeasureMeGlobal.ipAddress+"/measureme","postgres","measureme"); // CEH 9/23/2002

    this.initJavaSpaces();
    this.lookUpGeographyCodes();
    alm=new AutomatedLeaseManager(20000);
    frame = new NewAnalysisStationFrame(this);
    frame.addClientEntersStationListener(this);
    frame.addClientLeavesStationListener(this);
    this.clAdmin=new ClientAdministration(frame,null);
    this.clAdmin.addAdministrationTask(new ShutDownTask());
    frame.addAdminRequestListener(this.clAdmin);
    if(packFrame)
      frame.pack();
    else
      frame.validate();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = frame.getSize();
    if (frameSize.height > screenSize.height)
    { frameSize.height = screenSize.height;
    }
    if (frameSize.width > screenSize.width)
    { frameSize.width = screenSize.width;
    }
    frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    frame.setVisible(true);
    String scanCode = MeasureMeGlobal.scanCode;
    if (scanCode != null)
      try {
        login(scanCode);
      } catch(Exception e2) {
        System.out.println("Scan code " + scanCode + " not found in JavaSpace");
      }
  }
  public static void main(String args[]) {
    MeasureMeGlobal.init();
    try
    {
      //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      new NewAnalysisStation();
    }
    catch(Exception e)

    {
      e.printStackTrace();
    }
  }

  protected void initJavaSpaces()
  {
    statusDialog.setText("Searching for JavaSpace...");
    statusDialog.start();
    javaSpace=MMSpaceAccessor.getSpace();
    statusDialog.setText("Searching for Transaction Manager...");
    mgr=MMTransactionManagerAccessor.getManager();
    statusDialog.stop();
  }
/*
  /**
   * PURPOSE: pull the client tuple, and start a thread for the client if found
   */
  protected synchronized void login(String scanCode) throws UnknownClientException
  {
    if (this.theExhibitorWrapper != null)
    { if (scanCode.equals(this.theExhibitorWrapper.exhibitorScanID))
      {
        // logged in then got same code before logging out; ignore this one
        System.out.println("Got same scan code before log out; ignoring...");
        return;
      }
    }

    Transaction.Created trc= null; // a transaction that insures we won't lose
                                   // a client after we take it
    try{
      trc=TransactionFactory.create(this.mgr,60000);  //create a 60 second transactiion
      currentTransaction=trc.transaction;
    }
    catch(Exception e3)
    { System.err.println("Error obtaining transaction");
      return;
    }

    try{
      ExhibitorWrapper template=new ExhibitorWrapper();
      template.exhibitorScanID=scanCode;

     // edit M. Lowery 2001-06-06 changed from takeIfExists to take
     // edited by CEH on 10/23/2002 for optional read (no take)
      if (MeasureMeGlobal.takeTuple) // can be read now, rather than take
        theExhibitorWrapper=(ExhibitorWrapper)javaSpace.take(template,null,2000);
      else // read tuple -- useful in classroom setting
        theExhibitorWrapper=(ExhibitorWrapper)javaSpace.read(template,currentTransaction,10000);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      System.err.println(" Java space error in login method, resyncing.");
      this.recover();
      return; // CEH -- added 9/15/2002
      //this.login(scanCode); CEH -- changed 9/15/2002
    }

    if(theExhibitorWrapper==null)  //no exhibitor with that code in space
    { throw new UnknownClientException();
    }
    else
      System.out.println("Found client "+scanCode);

    alm.renewUntil(trc.lease,Lease.FOREVER,this);   //keep the transaction going indefinately
    this.currentLease=trc.lease;

    //this.frame.setCurrentAvailableStations(this.getLastExhibitAvailableInfo());

    this.frame.setExhibitor(this.theExhibitorWrapper.getExhibitor()); //
  }

  protected synchronized void logout()
  {
    try
    {
      if (MeasureMeGlobal.takeTuple) // can be read now, rather than take
        javaSpace.write(theExhibitorWrapper,currentTransaction,Lease.FOREVER);
      currentTransaction.commit();
      //javaSpace.write(theExhibitorWrapper,null,Lease.FOREVER);
      System.out.println("commited");
      theExhibitorWrapper = null;
      //alm.cancel(currentLease);
    }
    catch(Exception e)
    { e.printStackTrace();
    }
    frame.resetAll();
  }


  /**
   * PURPOSE: Try to recover in the case of a javaspace problem.
   */

  protected void recover()
  { try{
      this.mgr=MMTransactionManagerAccessor.getManager();
      this.javaSpace=MMSpaceAccessor.getSpace();
    }
    catch(Exception e)
    { e.printStackTrace();
    }

  }

  public void notify(LeaseRenewalEvent e)
  { System.out.println("Lease renewal: "+e);
    //TODO: Implement this edu.ucf.cs.dml.simnets.measureme.server.LeaseListener method
  }

  public void dataChanged(ClientEntersStationEvent e){
    String scanCode = (String)e.getSource();
    try{ this.login(scanCode);
    }
    catch(Exception e2)
    { System.out.println("Scan code " + scanCode + " not found in JavaSpace");
    }

  }

  public void dataChanged(ClientLeavesStationEvent e)
  { this.logout();

  }


  public QueryResults performQuery(QueryReplyRequest query)
  {
    //return null;
    return this.dbQE.handleDbQuery(query); // CEH 9/23/2002

  }

  /**
   * PURPOSE: find geography codes etc from the db
   */

  protected void lookUpGeographyCodes()
  {  // CEH 9/23/2002
      QueryResults result=this.dbQE.handleDbQuery(new QueryReplyRequest(
         "select * from countytable;","CountyCodes",true));

    ResultHolder rh=result.getResults();
    Vector v1=rh.getColumn(2);
    Vector v2=rh.getColumn(1);

    for(int i=0;i<v2.size();i++)
    { countyCodes.put(v1.elementAt(i),v2.elementAt(i) );

    }
    result=this.dbQE.handleDbQuery(new QueryReplyRequest(
         "select * from countrytable;","CountryCodes",true));
    rh=result.getResults();
    v1=rh.getColumn(2);
    v2=rh.getColumn(1);
    for(int i=0;i<v2.size();i++)
    { countryCodes.put(v1.elementAt(i),v2.elementAt(i) );
    }

  }

  /**
   *  Gets the last status of the other stations
   */


  protected Vector getLastExhibitAvailableInfo()
  { ExhibitAvailableQueue templateQ=new ExhibitAvailableQueue();
    templateQ.stationsAvailableInfo=null;
    ExhibitAvailableQueue theRealQ=null;
    try{
      theRealQ=(ExhibitAvailableQueue) this.javaSpace.readIfExists(templateQ,null,2000);
    }
    catch(Exception e)
    { e.printStackTrace();
    }
    if(theRealQ!=null)
    { System.out.println("Found exhibit info");
      return theRealQ.getAvailableStations();
    }
    System.err.println("Error: no available exhibit info");
    return null;

  }


    /**
   * PURPOSE: return the county mappings
   */

  public Hashtable getCountyTable()
  { return this.countyCodes;
  }

  /**
   * PURPOSE: return the country mappings
   */

  public Hashtable getCountryTable()
  { return this.countryCodes;
  }
}