package edu.ucf.cs.dml.simnets.measureme.client;

/**
 * Title:        Measure Me client side classes/source
 * Description:  The source code for the measure me orlando science center project.
 * Copyright:    Copyright (c) 1999
 * Company:      DML
 * @author Dean Reed
 * @version
 */

public class ArmSpanValueMap extends ValueMap {
  public static final float LOWEST_MEASUREMENT = 32.0F;
  public static final float DIST_BETWEEN_LEDS = 0.25F;

  public ArmSpanValueMap() {
    super();
  }

  public String getMetricUnits() {
    return "in";
  }

  /**
   * This function takes the value returned from getInternalValue and
   * converts it to a physical measurement.  Usually adding a constant
   * and multiplying by a constant.  Here 32" is the distance between
   * the arrays of LEDs.  0.25" is the distance between each LED.  Also,
   * we need to undo whatever getInternalValue did to the value.
   */
  public Object getActualValue(Short deviceValue) {
    return new Float(LOWEST_MEASUREMENT+((deviceValue.shortValue()+1)*DIST_BETWEEN_LEDS));
  }


  /**
   * This function takes the value returned from the device and
   * converts it to the way it will be stored in the database.
   * Max value is 256 so we subtract 1 to make it fit in the 0..255
   * range.
   */
  public Short getInternalValue(Object actualValue) {
    return new Short((short)(((Short) actualValue).shortValue()-1));
  }
}