package edu.ucf.cs.dml.simnets.measureme.client;

/**
 * Title:        Measure Me client side classes/source
 * Description:  The source code for the measure me orlando science center project.
 *               Encapsulates administration objects. Each administration task object may
 *               be added on the fly by use of addAdministrationTask or the overidden method
 *                configureAdministrationTasks which is called by the constructor.
 *               Note: This object must be added as a listener to an AdminRequest Event source
 * Copyright:    Copyright (c) 1999
 * Company:      DML
 * @author Dean Reed
 * @version
 */
import java.util.*;
import javax.swing.*;

public class ClientAdministration implements AdminRequestListener
{ protected Hashtable adminClasses=new Hashtable();
  protected JFrame gui=null;
  MeasureMeClient client=null;

  /**
   * Constructor: Provide the panel
   */
  public ClientAdministration(JFrame gui, MeasureMeClient client)
  { this.gui=gui;
    this.client=client;
    this.configureAdministrationTasks();

  }

  public void dataChanged(AdminRequestEvent e)
  {  String scanCode = (String)e.getSource();
     AdministrationTask task=(AdministrationTask)this.adminClasses.get(scanCode);
     if(task==null)
     { System.err.println("ERROR: request for non existant admin module");
       return;
     }
     if(task.getGui()==null)
      task.setGui(this.gui);
     Thread t=new Thread(task);
     t.start();

    /**@todo: Implement this edu.ucf.cs.dml.simnets.measureme.client.AdminRequestListener method*/
     // throw new java.lang.UnsupportedOperationException("Method dataChanged() not yet implemented.");
  }


  public void addAdministrationTask(AdministrationTask task)
  { if(task.getScanCode().equals(""))
    { System.err.println("Warning admin task with no scan id not added to the ClientAdministration");
      return;
    }
    this.addAdministrationTask(task,task.getScanCode());
  }

  public void addAdministrationTask(AdministrationTask task, String scanCode)
  { task.setScanCode(scanCode);
    this.adminClasses.put(scanCode,task);
  }



  /**
   * PURPOSE: Provide a mechanism for configuring automatically
   */
  protected void configureAdministrationTasks()
  { if(this.client!=null)
    {GreyCodeDeviceCalibration calib=new GreyCodeDeviceCalibration(this.client.getDeviceProxy());
     this.addAdministrationTask( calib);
    }
    //this.adminClasses

  }
}