
/**
 * Title:        Measure Me client side classes/source<p>
 * Description:  The source code for the measure me orlando science center project.<p>
 * Copyright:    Copyright (c) 1999<p>
 * Company:      DML<p>
 * @author Dean Reed
 * @version
 */
package edu.ucf.cs.dml.simnets.measureme.client;

import java.awt.*;
import com.borland.jbcl.control.*;
import com.borland.jbcl.layout.*;
import java.util.*;
import java.net.URL;
import java.lang.reflect.*;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;

/**
 * This class displays a histogram based on a dataset of 256 Shorts.  Also
 * it highlights the column where the current user's score falls.
 */
public class CurvePanel extends JPanel {

  /**
   * Height and width in pixels of images used as cells.
   */
  protected final int CELL_WIDTH = 16;
  protected final int CELL_HEIGHT = 16;

  /**
   * dataSet is array of size 256.  All 256 columns will not be shown in
   * histogram.  Instead we take groups of size 256/partitionCount and show
   * the totals in one column.
   */
  protected final int PARTITION_COUNT = 32;

  /**
   * Number of score slots which will display as one column in histogram.
   */
  protected final int PARTITION_SIZE = 256 / PARTITION_COUNT;

  /**
   * This array will contain the totals for each partition.  These numbers
   * will be used for each column.
   */
  protected Vector[] partitions = new Vector[PARTITION_COUNT];

  /**
   * Y-coordinate of cell starting point in pixels.  Cells are stacked on top
   * of each other starting here.
   */
  protected int CELL_BASE_LINE = 400; //the lowest y coordinate for cell drawing

  /**
   * Maximum number of cells per column.
   */
  protected int MAX_CELL_PER_COLUMN = 20;

  /**
   * Totals for each partition.
   */
  protected int[] partitionTotals = new int[PARTITION_COUNT];

  /**
   * Current data set.
   */
  protected Short[] dataSet = null;

  /**
   * This dataset is used if there are not enough visitors to create a decent
   * looking histogram.
   */
  protected Short fakeData[] = {
      new Short((short)1),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)2), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)3),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)4), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)5),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)6), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)7),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)8), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)9),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)10), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)11),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)12), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)13),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)14), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)15),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)16), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)16),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)15), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)14),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)13), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)12),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)11), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)10),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)9), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)8),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)7), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)6),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)5), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)4),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)3), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)2),new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
      new Short((short)1), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0), new Short((short)0),
  };

  /**
   * Current visitor's score.  Used to highlight correct column.
   */
  protected short currentScore = 0;

  /**
   * How many visitors each cell represents.
   */
  protected float scaleFactor=0f;

  /**
   * Highest partition count of all partitions.
   */
  protected int highestValue = 0;

  /**
   * Index of partition with highestValue.
   */
  protected int highestIndex = 0;

  /**
   * Used in labeling X axis.
   */
  protected ValueMap vMap = null;

  /**
   * Contains references to all cells that were added to this so that
   * we can remove them when we need to display new dataSet.
   */
  protected Vector imageLabels = new Vector();

  /**
   * This thread is instantiated and started whenever we need to repaint.
   */
  protected Thread myRunThread=null;

  /**
   * All registered listeners.
   */
  protected transient Vector curveDrawingCompletedListeners;

  /**
   * Variables of the object rather than local method variables since we use
   * an inner class to access them.  All of these variables have to do with
   * the highlighting rectangle.
   */
  protected int x1;
  protected int y1;
  protected int rectHeight;
  protected Graphics currentGraphics;
  protected javax.swing.Timer timer;
  protected boolean clearPass = true;

  /**
   * GUI stuff.
   */
  Color boxColor = Color.red;
  XYLayout xYLayout1 = new XYLayout();
  Image image1 = null;
  ImageIcon imageIcon1 = null;
  JLabel scaleImgLabel=null;
  JLabel scaleLabel=new JLabel();
  TitledBorder titledBorder1;
  RulerPanel rulerPanel1 = new RulerPanel();

  public CurvePanel() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  } // constructor

  void jbInit() throws Exception {
    titledBorder1 = new TitledBorder("");
    xYLayout1.setWidth(512);
    xYLayout1.setHeight(450);
    this.setBackground(SystemColor.info);
    this.setLayout(xYLayout1);
    this.scaleImgLabel=new JLabel(this.imageIcon1);
    this.scaleLabel.setFont(new java.awt.Font("SansSerif", 1, 16));
    this.add(this.scaleImgLabel, new XYConstraints(0, 0, -1, -1));
    this.add(this.scaleLabel, new XYConstraints(CELL_WIDTH+5, 0, -1, -1));
    this.add(rulerPanel1, new XYConstraints(0, 401, 512, 49));
    this.scaleLabel.setText("= "+ new Float(this.scaleFactor).toString() + " people.");
  } // jbInit

  /**
   * Computes histogram and displays.
   */
  public void setDataSet(Short[] newDataSet, short score) {
    currentScore = score;
    if (isLargeEnough(newDataSet)) {
      dataSet = newDataSet;
    }
    else {
      dataSet = fakeData;
    }
    computeNewDataSet(dataSet);
    clearAllIcons();
    Point p=null;
    JLabel tempLabel;
    // add each cell to screen

    for (int i = 0; i < PARTITION_COUNT; i++) {
      for(int j = 0; j < partitions[i].size(); j++) {
        p = (Point) partitions[i].get(j);
        tempLabel = new JLabel(this.imageIcon1);
        imageLabels.add(tempLabel);
        add(tempLabel, new XYConstraints(p.x, p.y, CELL_WIDTH, CELL_HEIGHT));
        tempLabel.setVisible(true);
      }
    }
    this.paintXAxisLabels(this.rulerPanel1.getGraphics());
    // let listeners know curve is done drawing
    System.out.println("firing curve done");
    fireCurveDone(new CurveDrawingCompletedEvent(this));
  } // setDataSet




  /**
   * Determines if current dataset is large enough to be considered normal.  Note
   * that totalVisitors is the total number of visitors to this station that have
   * age X.
   */
  protected boolean isLargeEnough(Short[] dataSet) {
    int totalVisitors=0;
    for(int i=0; i < dataSet.length; i++) {
      totalVisitors += dataSet[i].shortValue();
    }
    System.out.println("data set has "+ totalVisitors+ " score");
    return (totalVisitors >= 50);
  } // isLargeEnough


  /**
   * this method actually computes data in the set, and sets the vector of points
   * to the appropriate value.
   *
   */
  protected void computeNewDataSet(Short[] newSet) {

    // reset partitionTotals array
    for (int i = 0; i < PARTITION_COUNT; i++) {
      partitionTotals[i] = 0;
    }

    System.out.println("PARTITION_COUNT="+PARTITION_COUNT);
    System.out.println("PARTITION_SIZE="+PARTITION_SIZE);
    // calculate total people who scored within each partition
    highestValue = 0;
    highestIndex = 0;
    for (int i = 0; i < PARTITION_COUNT; i++) {
      for(int j = (PARTITION_SIZE*i); j < (PARTITION_SIZE*(i+1)); j++) {
        partitionTotals[i] += newSet[j].shortValue();
      }
      // remember largest partition score
      if (partitionTotals[i] > highestValue) {
        highestValue = partitionTotals[i];
        highestIndex = i;
      }
    }

    System.out.println("highestValue="+highestValue);
    System.out.println("highestIndex="+highestIndex);

    // calculate scale factor
    if (highestValue != 0) {
      scaleFactor= (float) highestValue / MAX_CELL_PER_COLUMN;
      System.out.println("scaleFactor="+scaleFactor);
    }

    // calculate points for each cell in each column
    int cellCount = 0;
    for (int i=0; i < PARTITION_COUNT; i++) {
      // clear out last set of points
      partitions[i] = new Vector();
      cellCount = (int) Math.round((float) partitionTotals[i] / scaleFactor);
      for(int j=0; j < cellCount; j++) {
        partitions[i].add(j,new Point(i * CELL_WIDTH,
                                      CELL_BASE_LINE - ((j+1) * CELL_HEIGHT)));
      }
    }
    // update scale factor on screen
    scaleLabel.setText("= " + scaleFactor + " people.");
  } // computeNewDataSet

  /**
   * Remove all cells from this JPanel.
   */
  protected void clearAllIcons() {
    System.out.println("removing old cells");
    for(int i=0;i<this.imageLabels.size();i++) {
      this.remove((JLabel)imageLabels.get(i));
    }
    this.imageLabels.removeAllElements();
    this.updateUI();
  } // clearAllIcons

  protected void paintComponent(Graphics g) {
    super.paintComponent(g);
    paintScoreCellBox((Graphics2D) g);
  }

  /**
   * Highlight column containing currentScore by encircling with flashing
   * rectangle.
   */
  protected void paintScoreCellBox(Graphics2D g) {
    currentGraphics = g;

    // find column where currentScore lands; truncating result will give 0-based
    // column number
    int partitionNumber = currentScore / PARTITION_SIZE;
    System.out.println("column where "+currentScore+" lands: " + partitionNumber);

    Vector v = partitions[partitionNumber];

    // remember original color
    Color originalColor = g.getColor();

    g.setColor(this.boxColor);
    g.setStroke(new BasicStroke(2));

/*
      timer = new javax.swing.Timer(1000, new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if ((clearPass = !clearPass) == true) {
          CurvePanel.this.updateUI();
        }
        else {
          currentGraphics.drawRoundRect(x1, y1, CELL_WIDTH, rectHeight, 8, 8);
        }
      }
    });
*/
    x1 = partitionNumber * CELL_WIDTH;
    y1 = CELL_BASE_LINE - (v.size() * CELL_HEIGHT);
    rectHeight = v.size() * CELL_HEIGHT;

    //timer.start();
    currentGraphics.drawRoundRect(x1, y1, CELL_WIDTH, rectHeight, 8, 8);
  } // paintScoreCellBox

  protected void paintXAxisLabels(Graphics g) {
    // x axis
    g.drawLine(0, CELL_BASE_LINE, PARTITION_COUNT*CELL_WIDTH, CELL_BASE_LINE);
    // y axis
    g.drawLine(0, CELL_BASE_LINE, 0, (int)(CELL_BASE_LINE - CELL_HEIGHT*MAX_CELL_PER_COLUMN));

    if (vMap != null) {
      int labelScale=32;
      Object[] valueLabels=new Object[8];

      int j=0;
      for( int i=0; i<256;i+=labelScale) {
        System.out.println(" X:"+this.vMap.getActualValue(new Short((short)(i))));
        valueLabels[j]=this.vMap.getActualValue(new Short((short)(i)));
        j++;
      }
      rulerPanel1.setScoreLabels(valueLabels, vMap.getMetricUnits());
      //axisDrawn=true;
    }
    else {
      System.out.println("Valuemap is null");
    }
  } // paintXAxisLabels

  public void setValueMap(ValueMap vMap) {
    this.vMap=vMap;
  } // setValueMap

  public ValueMap getValueMap() {
    return this.vMap;
  } // getValueMap

  /*
  protected void drawCurve(Graphics2D g2) {
    if(this.tallyScore==null)
     return;
    Field f=null;
    Vector currentVec=null;

    Class myClass=this.getClass();
    GeneralPath p=new GeneralPath(GeneralPath.WIND_NON_ZERO);
    Point beginPoint=null;
    Point endPoint=null;
    // points for the curve
    Point endPoint2=null;
     try{
        f=myClass.getDeclaredField("v1");
        currentVec=(Vector)f.get(this);
        if(currentVec.size()!=0)
          beginPoint=(Point)currentVec.get(currentVec.size()-1);
        else
          beginPoint=new Point(0,this.cellBaseLine-10);

        f=myClass.getDeclaredField("v"+new Integer(this.highestIndex+1).toString());
        currentVec=(Vector)f.get(this);
        endPoint=(Point)currentVec.get(currentVec.size()-1);

        f=myClass.getDeclaredField("v"+new Integer(this.numberOfDataPartition).toString());
        currentVec=(Vector)f.get(this);
        if(currentVec.size()>0)
         endPoint2=(Point)currentVec.get(currentVec.size()-1);
        else
         endPoint2=new Point(this.cellWidth*this.numberOfDataPartition,this.cellBaseLine);

        p.moveTo(beginPoint.x,beginPoint.y );
        p.curveTo(beginPoint.x,beginPoint.y,(beginPoint.x+endPoint.x)/2.5f ,(beginPoint.y+endPoint.y)/2.5f,endPoint.x+(this.cellWidth/2),endPoint.y);
        p.curveTo(endPoint.x+this.cellWidth+2,endPoint.y,(endPoint.x+endPoint2.x)/1.5f + 3,(endPoint.y+endPoint2.y )/1.5f -3,endPoint2.x,endPoint2.y);


        Stroke oldStroke=g2.getStroke();
        Color oldColor=g2.getColor();
        g2.setColor(Color.magenta);
        g2.setStroke(new BasicStroke(12.0f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_MITER));


        g2.draw(p);
        g2.setColor(oldColor);
        g2.setStroke(oldStroke);
         }
        catch(Exception e)
        { e.printStackTrace();
        }
    //figure out the highest point in the graph, and draw a bezier
  } // drawCurve
  */

  public synchronized void removeCurveDrawingCompletedListener(CurveDrawingCompletedListener l) {
    if (curveDrawingCompletedListeners != null && curveDrawingCompletedListeners.contains(l)) {
      Vector v = (Vector) curveDrawingCompletedListeners.clone();
      v.removeElement(l);
      curveDrawingCompletedListeners = v;
    }
  } // removeCurveDrawingCompletedListener

  public synchronized void addCurveDrawingCompletedListener(CurveDrawingCompletedListener l) {
    Vector v = curveDrawingCompletedListeners == null ? new Vector(2) : (Vector) curveDrawingCompletedListeners.clone();
    if (!v.contains(l)) {
      v.addElement(l);
      curveDrawingCompletedListeners = v;
    }
  } // addCurveDrawingCompletedListener

  protected void fireCurveDone(CurveDrawingCompletedEvent e) {
    if (curveDrawingCompletedListeners != null) {
      Vector listeners = curveDrawingCompletedListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++) {
        ((CurveDrawingCompletedListener) listeners.elementAt(i)).curveDone(e);
      }
    }
  } // fireCurveDone

  public void setAvatarImageIcon(ImageIcon icon) {
    this.imageIcon1=icon;
    this.scaleImgLabel.setIcon(icon);
  } // setAvatarImageIcon

/*
  public void stopThread() {

    try {timer.stop();} catch (NullPointerException e) {}
    if (timer.isRunning()) {
      System.out.println("timer still running after stop call");
    }
    else {
      System.out.println("timer stopped after stop call");
    }

    stopped = true;
    try {
      myRunThread.join();
    }
    catch (InterruptedException e) {
        e.printStackTrace();
        return;
    }
    catch (NullPointerException e2) {}
  } // stopThread
*/
} // CurvePanel