
/**
 * Title:        Measure Me client side classes/source<p>
 * Description:  The source code for the measure me orlando science center project.<p>
 *               This panel displays the current dataset to the user
 * Copyright:    Copyright (c) 1999<p>
 * Company:      DML<p>
 * @author Dean Reed
 * @version
 */
package edu.ucf.cs.dml.simnets.measureme.client;

import java.net.*;
import java.awt.*;
import com.borland.jbcl.layout.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;
import java.awt.event.*;

public class DisplayDataPanel extends JPanel implements CurveDrawingCompletedListener
{
  XYLayout xYLayout1 = new XYLayout();
  CurvePanel curvePanel1 = new CurvePanel();
  JLabel dataDisplayTextLabel = new JLabel();
  JLabel dataDisplayTextLabel2 = new JLabel();
  JLabel displayingAvatarAnimationLabel = new JLabel();
  TitledBorder titledBorder1;
  JButton yesBn = new JButton();
  JButton noBn = new JButton();
  long currentDisplayStart=0;
  private transient Vector clientEndsStationListeners;
  private transient Vector clientRestartsStationListeners;   // this time value will compensate for drawing
                                //speed and label display




  ImageIcon yesBnIcon=null;
  ImageIcon yesRolloverBnIcon=null;
  ImageIcon noBnIcon=null;
  ImageIcon noRolloverBnIcon=null;
  JLabel jLabel1 = new JLabel();
  JLabel scoreLabel = new JLabel();
  JLabel unitsLabel = new JLabel();
  protected short currentScore=0;
  protected String stationName = null;
  protected GuiParameters gp = null;

  public DisplayDataPanel(String stationName, GuiParameters gp)
  {
    this.stationName = stationName;
    this.gp = gp;
    try
    {

      yesBnIcon=new ImageIcon(new java.net.URL((URL)gp.getParameter("baseURL"),"buttons/yup.gif"));
      yesRolloverBnIcon=new ImageIcon(new java.net.URL((URL)gp.getParameter("baseURL"),"buttons/yup_rollover.gif"));
      noRolloverBnIcon=new ImageIcon(new java.net.URL((URL)gp.getParameter("baseURL"),"buttons/noperollover.gif"));
      noBnIcon=new ImageIcon(new java.net.URL((URL)gp.getParameter("baseURL"),"buttons/nope.gif"));
      jbInit();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  void jbInit() throws Exception
  {
    titledBorder1 = new TitledBorder("");
    xYLayout1.setWidth(800);
    xYLayout1.setHeight(600);
    this.setLayout(xYLayout1);
    dataDisplayTextLabel.setFont(new java.awt.Font("SansSerif", 1, 20));
    dataDisplayTextLabel2.setFont(new java.awt.Font("SansSerif", 1, 20));
    this.dataDisplayTextLabel2.setText("Good job. Do you want to try again?");
    displayingAvatarAnimationLabel.setBorder(BorderFactory.createLoweredBevelBorder());
    yesBn.setFont(new java.awt.Font("Serif", 0, 20));
    yesBn.setForeground(Color.green);
    yesBn.setRequestFocusEnabled(false);
    yesBn.setIcon(yesBnIcon);
    yesBn.setRolloverEnabled(true);
    yesBn.setRolloverIcon(yesRolloverBnIcon);

    yesBn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        // Edit M. Lowery disable button so can't generate any more events
        yesBn.setEnabled(false);
        yesBn_actionPerformed(e);
      }
    });
    noBn.setFont(new java.awt.Font("Serif", 0, 20));
    noBn.setForeground(Color.lightGray);
    noBn.setRequestFocusEnabled(false);
    noBn.setIcon(noBnIcon);
    noBn.setRolloverEnabled(true);
    noBn.setRolloverIcon(noRolloverBnIcon);
    noBn.addActionListener(new java.awt.event.ActionListener()
    {

      public void actionPerformed(ActionEvent e)
      {
        // Edit M. Lowery disable button so can't generate any more events
        noBn.setEnabled(false);
        noBn_actionPerformed(e);
      }
    });
    curvePanel1.setBackground(Color.cyan);
    jLabel1.setFont(new java.awt.Font("SansSerif", 1, 28));
    jLabel1.setText("Your score:");
    scoreLabel.setFont(new java.awt.Font("Dialog", 1, 28));
    unitsLabel.setFont(new java.awt.Font("SansSerif", 1, 28));
    this.add(dataDisplayTextLabel, new XYConstraints(20, 10, -1, -1));
    this.add(dataDisplayTextLabel2, new XYConstraints(20, 35, -1, -1));
    this.add(displayingAvatarAnimationLabel, new XYConstraints(44, 116, -1, -1));
    this.add(curvePanel1, new XYConstraints(289, 116, -1, -1));
    this.add(noBn, new XYConstraints(545, 77, 256, 39));
    this.add(yesBn, new XYConstraints(288, 77, 256, 39));
    this.add(jLabel1, new XYConstraints(44, 360, -1, -1));
    this.add(scoreLabel, new XYConstraints(44, 403, -1, -1));
    this.add(unitsLabel, new XYConstraints(44, 442, -1, -1));
  }

  public void setAvatarIcon(ImageIcon avatarIcon)
  { ImageIcon icon=(ImageIcon)this.displayingAvatarAnimationLabel.getIcon();
    if(icon!=null)
     if( icon.getImage()!=null)
       icon.getImage().flush();
    this.displayingAvatarAnimationLabel.setIcon(avatarIcon);
  }


  public void setGraphingIcon(ImageIcon graphingIcon)
  { System.out.println("setting new graphing icon");
    this.curvePanel1.setAvatarImageIcon(graphingIcon);
  }

  /**
   * PURPOSE: display the latest dataset, and highlight the area
   *  with a given score.
   */
  public void displayNewDataSet(Short[] dataSet, short score, short age)
  { //figure out the textline
    this.currentScore=score;
    this.yesBn.setVisible(false);
    this.noBn.setVisible(false);
    this.yesBn.setEnabled(false);
    this.noBn.setEnabled(false);
    //this.dataDisplayTextLabel2.setText("");

    this.curvePanel1.addCurveDrawingCompletedListener(this);
    //float scorePerc=(float)score/255f;

    long didBetterThanCount = 0;
    long visitorCount = 0;

    // Mat Edit 2001-02-29 changed the way the percentage was calculated



    System.out.println("DisplayDataPanel.displayNewDataSet(): dataSet length: "+dataSet.length);
    System.out.print("[");
    for (int j = 0; j < dataSet.length; j++) {
      if (j != 0) {
        System.out.print(", "+dataSet[j]);
      }
      else {
        System.out.print(dataSet[j]);
      }

    }
    System.out.println("]");


    System.out.println("DisplayDataPanel.displayNewDataSet(): score: "+score);

    /** Hack */
    if(this.stationName.equals("Reaction Time")) {
      for (int i = dataSet.length-1; i >= 0; i--) {
        if (i == score) {
          didBetterThanCount = visitorCount;
        }
        visitorCount += dataSet[i].shortValue();
      }

    }
    else {
      // count the total visitors at this station
      //    noting count at current user's score index
      for (int i = 0; i < dataSet.length; i++) {
        if (i == score) {
          didBetterThanCount = visitorCount;
        }
        visitorCount += dataSet[i].shortValue();
      }
    }

    double scorePerc;
    if (visitorCount != 0) {
      scorePerc = (double) didBetterThanCount / (double) visitorCount;
    }
    else {
      scorePerc = 1;    // 100% if no one else has ever been at station
    }

    System.out.println("DisplayDataPanel.displayNewDataSet(): didBetterThanCount: "+didBetterThanCount);
    System.out.println("DisplayDataPanel.displayNewDataSet(): visitorCount: "+visitorCount);
    System.out.println("DisplayDataPanel.displayNewDataSet(): scorePerc: "+scorePerc);


    //System.out.println("Score was: " +score+ "Score perc (score/256) was "+scorePerc );
    if(this.curvePanel1.getValueMap()!=null)
    { String scVal= this.curvePanel1.getValueMap().getActualValue(new Short(score)).toString();
      if(scVal.length() >5) //format
       scVal=scVal.substring(0,5);
      this.scoreLabel.setText(scVal);
      if(this.curvePanel1.getValueMap().getMetricUnits().equals("s"))
       this.unitsLabel.setText("seconds");
      else
       this.unitsLabel.setText(this.curvePanel1.getValueMap().getMetricUnits());
    }
    String tmp = (String) gp.getParameter("displayDataLabel");
    if (tmp != null) {
      this.dataDisplayTextLabel.setText(tmp+" "+Math.round(scorePerc*100)+"% of your age group!");
    }
    else {
      this.dataDisplayTextLabel.setText("Wow! You scored better than "+Math.round(scorePerc*100)+"% of your age group!");
    }
    this.currentDisplayStart=System.currentTimeMillis();

    this.curvePanel1.setDataSet(dataSet,score);
  }

  public void setValueMap(ValueMap vMap)
  { this.curvePanel1.setValueMap(vMap);
  }


  public void curveDone(CurveDrawingCompletedEvent e)
  {


/*     try{
      while(System.currentTimeMillis()-this.currentDisplayStart < 12500)
       Thread.sleep(200);
     }
     catch(Exception e2)
     { e2.printStackTrace();
     }
*/

    this.yesBn.setVisible(true);
    this.noBn.setVisible(true);
    this.yesBn.setEnabled(true);
    this.noBn.setEnabled(true);


    this.curvePanel1.removeCurveDrawingCompletedListener(this);
  }

  public synchronized void removeClientEndsStationListener(ClientEndsStationListener l)
  {
    if (clientEndsStationListeners != null && clientEndsStationListeners.contains(l))
    {
      Vector v = (Vector) clientEndsStationListeners.clone();
      v.removeElement(l);
      clientEndsStationListeners = v;
    }
  }

  public synchronized void addClientEndsStationListener(ClientEndsStationListener l)
  {
    Vector v = clientEndsStationListeners == null ? new Vector(2) : (Vector) clientEndsStationListeners.clone();
    if (!v.contains(l))
    {
      v.addElement(l);
      clientEndsStationListeners = v;
    }
  }

  protected void fireDataChanged(ClientEndsStationEvent e)
  {


    if (clientEndsStationListeners != null)
    {
      Vector listeners = clientEndsStationListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++)
      {
        ((ClientEndsStationListener) listeners.elementAt(i)).dataChanged(e);
      }
    }
  }

  public synchronized void removeClientRestartsStationListener(ClientRestartsStationListener l)
  {
    if (clientRestartsStationListeners != null && clientRestartsStationListeners.contains(l))
    {
      Vector v = (Vector) clientRestartsStationListeners.clone();
      v.removeElement(l);
      clientRestartsStationListeners = v;
    }
  }

  public synchronized void addClientRestartsStationListener(ClientRestartsStationListener l)
  {
    Vector v = clientRestartsStationListeners == null ? new Vector(2) : (Vector) clientRestartsStationListeners.clone();
    if (!v.contains(l))
    {
      v.addElement(l);
      clientRestartsStationListeners = v;
    }
  }

  protected void fireDataChanged(ClientRestartsStationEvent e)
  {
    if (clientRestartsStationListeners != null)
    {
      Vector listeners = clientRestartsStationListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++)
      {
        ((ClientRestartsStationListener) listeners.elementAt(i)).dataChanged(e);
      }
    }
  }

  void yesBn_actionPerformed(ActionEvent e)
  {
     this.fireDataChanged(new ClientRestartsStationEvent(this));

  }

  void noBn_actionPerformed(ActionEvent e)
  {
    this.fireDataChanged(new ClientEndsStationEvent(this));

  }

  public void addCurveDrawingCompletedListener(CurveDrawingCompletedListener l)
  { this.curvePanel1.addCurveDrawingCompletedListener(l);
  }

   public void removeCurveDrawingCompletedListener(CurveDrawingCompletedListener l)
   { this.curvePanel1.removeCurveDrawingCompletedListener(l);

   }




  public short getCurrentScore()
  { return this.currentScore;

  }



}