package edu.ucf.cs.dml.simnets.measureme.client;

/**
 * Title:        Measure Me client side classes/source
 * Description:  The source code for the measure me orlando science center project.
 * Copyright:    Copyright (c) 1999
 * Company:      DML
 * @author Mathew Lowery
 * @version
 */

/**
 * Device range is 22.5 inches.  There are 256 positions of the encoder.
 */
public class FlexibilityValueMap extends ValueMap {
  public static final float LOWEST_MEASUREMENT = 0.0F;
  public static final float HIGHEST_MEASUREMENT = 22.5F;
  public static final float INCHES_PER_BIT = 22.5F / 256;

  public FlexibilityValueMap() {
    super();
  }

  public String getMetricUnits() {
    return "in";
  }

  /**
   * Get value in inches for display to user.
   */
  public Object getActualValue(Short deviceValue) {
    return new Float(INCHES_PER_BIT * deviceValue.shortValue());
  }

  /**
   * Get internal value which will be stored in database.
   */
  public Short getInternalValue(Object actualValue) {
    Float f = null;
    float bitsPerInch = 1 / INCHES_PER_BIT;
    if (actualValue instanceof Float) {
      f = (Float) actualValue;
    }
    else {
      f=new Float(((Float) this.getActualValue((Short)actualValue)).floatValue() );
    }

    if(f.floatValue()<LOWEST_MEASUREMENT)
     return new Short((short)0);
    if(f.floatValue() > HIGHEST_MEASUREMENT)
     return new Short((short)255);
    Short s=new Short((short) Math.round(bitsPerInch * f.floatValue()));
    if(s.shortValue()<256)
     return s;
    return new Short((short)255);

  }

} // FlexibilityValueMap