package edu.ucf.cs.dml.simnets.measureme.client;

import edu.ucf.cs.dml.simnets.measureme.devices.*;
import java.awt.*;
import javax.swing.*;
import com.borland.jbcl.layout.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/**
 * Title:        Measure Me client side classes/source
 * Description:  The source code for the measure me orlando science center project.
 * Copyright:    Copyright (c) 1999
 * Company:      DML
 * @author Dean Reed
 * @version
 */

public class GreyCodeDeviceCalibrationFrame extends JDialog
{
  JPanel panel1 = new JPanel();
  XYLayout xYLayout1 = new XYLayout();
  XYLayout xYLayout2 = new XYLayout();
  JLabel jLabel1 = new JLabel();
  CalculatorInputPanel calculatorInputPanel1 = new CalculatorInputPanel();
  JLabel instructionsLabel = new JLabel();
  JLabel currentLowLabel = new JLabel();
  JLabel jLabel2 = new JLabel();
  JButton autoButton = new JButton();
  JButton manualButton = new JButton();
  JButton saveAndQuitButton = new JButton();
  protected boolean autoCalibrate=false;
  protected boolean manCalibrate=false;



  private transient Vector calibrationStateOptionListeners;
  JButton okBn = new JButton();

  public GreyCodeDeviceCalibrationFrame(Frame frame, String title, boolean modal)
  {
    super(frame, title, modal);
    try
    {
      jbInit();
      pack();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
    }
  }

  public GreyCodeDeviceCalibrationFrame()
  {
    this(null, "", false);
  }
  void jbInit() throws Exception
  {
    panel1.setLayout(xYLayout1);
    this.getContentPane().setLayout(xYLayout2);
    xYLayout2.setWidth(547);
    xYLayout2.setHeight(298);
    jLabel1.setFont(new java.awt.Font("Dialog", 1, 16));
    jLabel1.setBorder(BorderFactory.createEtchedBorder());
    jLabel1.setRequestFocusEnabled(false);
    jLabel1.setText("Grey Code Calibration Utility");
    instructionsLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
    instructionsLabel.setText("Move the device to lowest position.");
    currentLowLabel.setFont(new java.awt.Font("Dialog", 1, 14));
    currentLowLabel.setText("0");
    jLabel2.setFont(new java.awt.Font("SansSerif", 1, 14));
    jLabel2.setToolTipText("The zero value is the value of the device in its lowest position. ");
    jLabel2.setText("Zero Value:");
    autoButton.setText("Auto ");
    autoButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        autoButton_actionPerformed(e);
      }
    });
    manualButton.setText("Manual");
    manualButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        manualButton_actionPerformed(e);
      }
    });
    saveAndQuitButton.setEnabled(false);
    saveAndQuitButton.setText("Save + Quit");
    saveAndQuitButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        saveAndQuitButton_actionPerformed(e);
      }
    });
    okBn.setEnabled(false);
    okBn.setText("OK");
    okBn.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        okBn_actionPerformed(e);
      }
    });
    this.setModal(true);
    this.setTitle("Calibration");
    calculatorInputPanel1.addCalculatorInputFinishedListener(new edu.ucf.cs.dml.simnets.measureme.client.CalculatorInputFinishedListener()
    {
      public void dataChanged(CalculatorInputFinishedEvent e)
      {
        calculatorInputPanel1_dataChanged(e);
      }
    });
    this.getContentPane().add(panel1, new XYConstraints(0, 0, 546, 297));
    panel1.add(jLabel1, new XYConstraints(167, 10, -1, -1));
    panel1.add(calculatorInputPanel1, new XYConstraints(354, 29, -1, -1));
    panel1.add(instructionsLabel, new XYConstraints(24, 55, -1, -1));
    panel1.add(jLabel2, new XYConstraints(24, 115, -1, -1));
    panel1.add(currentLowLabel, new XYConstraints(108, 115, -1, -1));
    panel1.add(okBn, new XYConstraints(215, 84, -1, -1));
    panel1.add(autoButton, new XYConstraints(100, 235, -1, -1));
    panel1.add(manualButton, new XYConstraints(210, 235, -1, -1));
    panel1.add(saveAndQuitButton, new XYConstraints(334, 235, -1, -1));
    //this.calculatorInputPanel1.setVisible(false);
  }

  protected void initialState()
  { this.instructionsLabel.setText("Choose a calibration option below");
    this.calculatorInputPanel1.setVisible(false);
    this.saveAndQuitButton.setEnabled(false);
    this.okBn.setEnabled(false);
    this.manCalibrate=false;
    this.autoCalibrate=false;


  }

  void autoButton_actionPerformed(ActionEvent e)
  { this.instructionsLabel.setText("Move the Device to the (zero) position, and hit OK");
    this.autoCalibrate=true;
    this.okBn.setEnabled(true);

  }

  public void setHighValue(int high)
  {// this.currentHighValue.setText(Integer.toString(high));
  }

  public void setLowValue(int low)
  { this.currentLowLabel.setText(Integer.toString(low));
  }

  public synchronized void removeCalibrationStateOptionListener(CalibrationStateOptionListener l)
  {
    if (calibrationStateOptionListeners != null && calibrationStateOptionListeners.contains(l))
    {
      Vector v = (Vector) calibrationStateOptionListeners.clone();
      v.removeElement(l);
      calibrationStateOptionListeners = v;
    }
  }
  public synchronized void addCalibrationStateOptionListener(CalibrationStateOptionListener l)
  {
    Vector v = calibrationStateOptionListeners == null ? new Vector(2) : (Vector) calibrationStateOptionListeners.clone();
    if (!v.contains(l))
    {
      v.addElement(l);
      calibrationStateOptionListeners = v;
    }
  }
  protected void fireDataChanged(CalibrationStateOptionEvent e)
  {
    if (calibrationStateOptionListeners != null)
    {
      Vector listeners = calibrationStateOptionListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++)
      {
        ((CalibrationStateOptionListener) listeners.elementAt(i)).dataChanged(e);
      }
    }
  }

  void okBn_actionPerformed(ActionEvent e)
  {


    if(this.autoCalibrate)
    { //this.autoCalibrate=false;
      this.fireDataChanged(new CalibrationStateOptionEvent("Low"));
      this.saveAndQuitButton.setEnabled(true);
      this.instructionsLabel.setText("Now press the save + quit button. Then restart the station.");
    }


    if(this.manCalibrate)
    { //this.manCalibrate=false;
      this.setLowValue(this.calculatorInputPanel1.getCalculatorInput());
      this.saveAndQuitButton.setEnabled(true);
      this.instructionsLabel.setText("Now press the save + quit button");
    }





  }

  void manualButton_actionPerformed(ActionEvent e)
  { this.instructionsLabel.setText("Enter the number for the low and press ok.");
    this.calculatorInputPanel1.setVisible(true);
    this.manCalibrate=true;
    this.okBn.setEnabled(true);
  }

  void saveAndQuitButton_actionPerformed(ActionEvent e)
  { File f=new File("C:\\measureme\\greycodecalibration.ser");
    try{
     FileOutputStream fos=new FileOutputStream(f);
     ObjectOutputStream oos=new ObjectOutputStream(fos);
     GrayCodeCalibrationData data= new GrayCodeCalibrationData();
     data.setZeroValue((short)this.getModulo());

     oos.writeObject(data);
     oos.close();
    }
    catch(Exception e2)
    { this.instructionsLabel.setText("Error on file write!");

    }
    this.setVisible(false);
  }

  protected int getModulo()
  { Integer val=new Integer(this.currentLowLabel.getText());

    return val.intValue();
  }

  void calculatorInputPanel1_dataChanged(CalculatorInputFinishedEvent e)
  { this.setLowValue(this.calculatorInputPanel1.getCalculatorInput());
    this.instructionsLabel.setText("Now press the save + quit button. Then restart the station.");
    this.calculatorInputPanel1.setVisible(false);
    this.saveAndQuitButton.setEnabled(true);


  }



}