
/**
 * Title:        Analysis Station<p>
 * Description:  Analysis Station Code<p>
 * Copyright:    Copyright (c) Dean Reed<p>
 * Company:      IST/DML<p>
 * @author Dean Reed
 * @version 1.0
 */
package edu.ucf.cs.dml.simnets.measureme.client;
import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.*;
import edu.ucf.cs.dml.simnets.measureme.*;

/**
 * CLASS KeyCodeHandler: Manages key events, and fires events when specific codes for
 * MeasureMe are encountered including administrative events and login events.
 */
public class KeyCodeHandler implements KeyListener
{ public static String NULL_CODE="0000";
  public static String QUIT_CODE="114";
  public static int ADMIN_LOWER=1;
  public static int ADMIN_UPPER=999;
  public static int USER_LOWER=1000;
  public static int USER_UPPER=9999;
  public static int TERMINAL_CODE=10;   //terminates end of line
  protected String scanString="";
  protected String lastScanLoggedin="";
  private transient Vector clientEntersStationListeners;
  private transient Vector clientLeavesStationListeners;
  private transient Vector adminRequestListeners;
  protected int j = 0;


  public KeyCodeHandler(Frame listenComponent)
  {System.out.println("Mat - KeyCodeHandler constructed !!!!!!");

   listenComponent.addKeyListener(this);
  }
   //ignored1
  public void keyTyped(KeyEvent e)
  {

  }


  public void keyPressed(KeyEvent e) {
    System.out.println("Key: "+ e.getKeyChar() + " Code:"+e.getKeyCode());
    if(e.getKeyCode()==TERMINAL_CODE) {  //end of the line
      //time to analyze our string and fire appropriate events
      System.out.println("Terminal");

      if (this.scanString.length() == 0) {
        return;
      }

      if(this.scanString.equals(NULL_CODE) || this.scanString.equals(QUIT_CODE)) {
        this.fireDataChanged(new ClientLeavesStationEvent(new String(this.lastScanLoggedin)));
        this.lastScanLoggedin="";

      }
      if (this.scanString.equals(NULL_CODE))
        this.scanString="";
      else {
        try {
          j = Integer.parseInt(this.scanString);
        }
        catch (NumberFormatException e1) {
          // number too long; clear the scanString
          System.out.println("keyCodeHandler error: scanString="+scanString);
          scanString = "";
          return;
        }

        if((j >=USER_LOWER) && (j <=USER_UPPER)) {
          // a new user
          this.fireDataChanged(new ClientEntersStationEvent(new String(this.scanString)));
          this.lastScanLoggedin=this.scanString;
          this.scanString="";
        }
        else {
          if((j >=ADMIN_LOWER) && (j <=ADMIN_UPPER) ) {
            //an administrative code
            // fire admin event
            if (scanString.equals(QUIT_CODE))
              System.out.println("Got quit code; should stop now");
            this.fireDataChanged(new AdminRequestEvent(new String(this.scanString)));
            this.scanString="";
            MeasureMeGlobal.shuttingDown = true;
          }
          else {
            System.err.println("Error, string : "+this.scanString+" not recognized");
            this.scanString="";
          }
        }
      }
    }
    else {
      if((e.getKeyChar()>='0') && (e.getKeyChar()<='9')) {
        // a valid number add it to our string
        this.scanString+=e.getKeyChar();
      }
    }
  }

  //ignored
  public void keyReleased(KeyEvent e)
  { }

  public synchronized void removeClientEntersStationListener(ClientEntersStationListener l)
  {
    if (clientEntersStationListeners != null && clientEntersStationListeners.contains(l))
    {
      Vector v = (Vector) clientEntersStationListeners.clone();
      v.removeElement(l);
      clientEntersStationListeners = v;
    }
  }

  public synchronized void addClientEntersStationListener(ClientEntersStationListener l)
  {
    Vector v = clientEntersStationListeners == null ? new Vector(2) : (Vector) clientEntersStationListeners.clone();
    if (!v.contains(l))
    {
      v.addElement(l);
      clientEntersStationListeners = v;
    }
  }

  protected void fireDataChanged(ClientEntersStationEvent e)
  {
    if (clientEntersStationListeners != null)
    {
      Vector listeners = clientEntersStationListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++)
      {
        ((ClientEntersStationListener) listeners.elementAt(i)).dataChanged(e);
      }
    }
  }

  public synchronized void removeClientLeavesStationListener(ClientLeavesStationListener l)
  {
    if (clientLeavesStationListeners != null && clientLeavesStationListeners.contains(l))
    {
      Vector v = (Vector) clientLeavesStationListeners.clone();
      v.removeElement(l);
      clientLeavesStationListeners = v;
    }
  }

  public synchronized void addClientLeavesStationListener(ClientLeavesStationListener l)
  {
    Vector v = clientLeavesStationListeners == null ? new Vector(2) : (Vector) clientLeavesStationListeners.clone();
    if (!v.contains(l))
    {
      v.addElement(l);
      clientLeavesStationListeners = v;
    }
  }

  protected void fireDataChanged(ClientLeavesStationEvent e)
  {
    if (clientLeavesStationListeners != null)
    {
      Vector listeners = clientLeavesStationListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++)
      {
        ((ClientLeavesStationListener) listeners.elementAt(i)).dataChanged(e);
      }
    }
  }

  public synchronized void removeAdminRequestListener(AdminRequestListener l)
  {
    if (adminRequestListeners != null && adminRequestListeners.contains(l))
    {
      Vector v = (Vector) adminRequestListeners.clone();
      v.removeElement(l);
      adminRequestListeners = v;
    }
  }

  public synchronized void addAdminRequestListener(AdminRequestListener l)
  {
    Vector v = adminRequestListeners == null ? new Vector(2) : (Vector) adminRequestListeners.clone();
    if (!v.contains(l))
    {
      v.addElement(l);
      adminRequestListeners = v;
    }
  }

  protected void fireDataChanged(AdminRequestEvent e)
  {
    if (adminRequestListeners != null)
    {
      Vector listeners = adminRequestListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++)
      {
        ((AdminRequestListener) listeners.elementAt(i)).dataChanged(e);
      }
    }
  }
}