package edu.ucf.cs.dml.simnets.measureme.client;

import edu.ucf.cs.dml.simnets.measureme.MeasureMeGlobal;
import edu.ucf.cs.dml.simnets.measureme.server.*;
import edu.ucf.cs.dml.simnets.measureme.devices.*;
import edu.ucf.cs.dml.simnets.measureme.dbase.*;
import javax.swing.UIManager;
import java.awt.*;
import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;
import java.net.*;
import java.util.*;
import java.io.*;
import  com.borland.jb.io.AsciiInputStream;
import edu.ucf.cs.dml.simnets.measureme.*;

public class MeasureMeClient implements ClientEntersStationListener, ClientLeavesStationListener, ClientEnteredDemographicsListener, ClientStartsMeasurementListener, ClientRestartsStationListener, ClientEndsStationListener
{ protected MajorBukuError mbe=new MajorBukuError(); //used in case no java space, and no params
  boolean packFrame = false;

  protected MeasureMeFrame mainGui=null;              //the gui

  protected String paramFile="stationparams.ser"; //paramter storage
  protected String codeFile="stationcodes.ser";   //codes for country, county storage
  protected String stationSetup ="station.txt";   // a text file with the name of the station
  protected String oldStationDataFile="olddata.ser"; // save a backup of old paramters


  protected PrintStream debug;

  protected AvatarInformation avatarInf=new AvatarInformation(); //hard coded avatar data
  protected StationParameters stationParameters=null;           //the paramters for the station
  protected JavaSpace javaSpace=null;
  protected JavaSpacesStatus jsStatus=null;                    //displays connection status

  protected String stationName=null;                           //station name i.e. Hang Time
  protected short stationIDCode=-1;                            //this stations id code 1..25

  protected Device myDevice=null;                              //the device object for this station
  protected DeviceProxy myDeviceProxy=null;                    //a high level proxy
  protected ValueMap stationValueMap=null;                     //maps from device to 0..255


  protected Exhibitor currentExhibitor=null;                   //exhibitor in the station
  protected Thread eThread=null;                      //controls flow of the exhibitor
  //protected boolean clientLeft = false;

  protected DemographicDecisions demDecision=null;             //demographic logic
  protected Hashtable stationAgeScores = null;                 //the age data for this station

  //protected boolean deviceActivated=false;                     //does the device signal data collection?



  protected Hashtable countryCodes=null;                       //codes used in the
  protected Hashtable countyCodes=null;                        //database for country, county
  protected boolean restartRequest=false;                       //has the user requested a restart?
  protected ExhibitAvailableManager eam=null;                  //checks to determine which exhibits are available

  protected ClientAdministration clAdmin=null;                 // used for station admin




  /**
   * INNERCLASS ExhibitorThread: A runnable class that handles the exhibitor through
   * the exhibit. A new thread should start when an exhibitor enters the exhibit.
   */
  protected class ExhibitorThread implements Runnable {
    public ExhibitorThread(String scanCode) throws Exception {
      MeasureMeGlobal.clientLeft = false;
      ExhibitorWrapper template=new ExhibitorWrapper();
      template.exhibitorScanID=scanCode;
      ExhibitorWrapper theExhibitorWrapper=(ExhibitorWrapper)javaSpace.take(template,null,2000);
      if(theExhibitorWrapper==null)
      { System.out.println("no client in space, creating a new client");
        theExhibitorWrapper=new ExhibitorWrapper(currentExhibitor=new Exhibitor(scanCode));
        ExhibitScoreInfo inf=new ExhibitScoreInfo(stationIDCode); //add this stations data
        inf.setFirstVisitTime(new java.util.Date());
        currentExhibitor.addExhibitScoreInfo(inf);                //set start time for client
      }
      else
      { System.out.println("found a matching entry");
        currentExhibitor=theExhibitorWrapper.getExhibitor();
        //check to see if this exhibitor has visited before...
        ExhibitScoreInfo inf=currentExhibitor.getScoreInfo(stationIDCode);
        if(inf==null) //never visited this station before
        { inf=new ExhibitScoreInfo(stationIDCode);
          currentExhibitor.addExhibitScoreInfo(inf);
        }
        inf.setFirstVisitTime(new java.util.Date()); //set start time for client
      }
    }


    /**
     * The run method is signaled to quit by setting the flag var clientLeft
     * to true.  It should be checked often in this method.  When this method
     * returns, the client thread will die.
     */
    public void run() {
      do {
        restartRequest=false;
        if(!demDecision.getDemographics(currentExhibitor))//unable to collect demographics
        {
          if(MeasureMeGlobal.clientLeft) //client leaves without any collection
          {
            break;
          }
          else {
            System.out.println("Error.... return false from demograph, client still here");
          }
        }



          if (!MeasureMeGlobal.clientLeft) {
            //demographics ok; time to show teaching panel for this exhibit
            mainGui.flipToTeaching();
          }

          System.out.println("Waiting for device signal");
          // block until first data arrives from device
          if (!MeasureMeGlobal.clientLeft) {
            try {
              myDeviceProxy.waitForDeviceMeasure();
            }
            catch(InterruptedException e1) {
              e1.printStackTrace();
              return;
            }
            catch (DeviceFailureException e2) {
              e2.printStackTrace();
            }
          }


          /**
           * deviceActivated determines what we do at this point.  If true, this
           * means the device has a collecting data stage where an animation
           * is displayed while we're collecting data.  For example, on Hang Time,
           * deviceActivated==true and waitForDeviceMeasure will block until we
           * receive the first data byte.  This is in contrast to Foot Length,
           * which does not have an animation and only stores the last data value
           * sent from the device.
           */

          if(MeasureMeGlobal.deviceActivated)
          {


            // display collecting data animation
            if (!MeasureMeGlobal.clientLeft) {
              mainGui.flipToCollectData();
            }

            // holds the actual score or measurement
            Object measureVal=null;

            // block until client done with activity (i.e. done hanging, etc.)
            System.out.println("calling deviceproxy.measure...");
            if(!MeasureMeGlobal.clientLeft)
            {
              try {
                measureVal=myDeviceProxy.measure();
              }
              catch(InterruptedException e1) {
                e1.printStackTrace();
                return;
              }
              catch (DeviceFailureException e2) {
                e2.printStackTrace();
              }
            }
            //System.out.println("Measure Object: "+measureVal );
            //System.out.println("maps to an internal value of: "+stationValueMap.getInternalValue(measureVal).shortValue() );
            if (!MeasureMeGlobal.clientLeft) {
              if(stationParameters.getStationName().equals("Hang Time"))
              {
                currentExhibitor.setWeight((short) Math.round( ((HangTimeValue)measureVal).getWeight()) );
                mainGui.setWeight(((HangTimeValue)measureVal).getWeight());
              }
            }

            //the measurement is complete, now add to local data
            if (!MeasureMeGlobal.clientLeft) {
              addToCurrentUserScore(stationValueMap.getInternalValue(measureVal));
            }

            if (!MeasureMeGlobal.clientLeft) {
              mainGui.showResults((Short[])stationAgeScores.get(new Short(currentExhibitor.getAge())),stationValueMap.getInternalValue(measureVal).shortValue(),currentExhibitor.getAge());
            }
          } // otherwise, the client enters measurement event is generated


        if(!MeasureMeGlobal.clientLeft) {
          while(!MeasureMeGlobal.clientLeft && !restartRequest) {
            try {
              System.out.println("waiting for client to leave or restart");
              synchronized (eThread) {
                eThread.wait(MeasureMeGlobal.WAIT_TIME);
              }
            }
            catch(InterruptedException e3) {
              e3.printStackTrace();
              return;
            }
          }
        }
     }// do-while
     while (restartRequest&&!MeasureMeGlobal.clientLeft);
     System.out.println("run method of MeasureMeClient returning...");
    }



  }

  //Construct the application
  public MeasureMeClient() {
    boolean newParams=false;
    getTextFile();                            //get the local configuration data
    //myDevice=this.loadDevice(deviceName);

    jsStatus=new JavaSpacesStatus();  // lookup mode status
    jsStatus.setVisible(true);
    lookup();                        //perform the actual java spaces lookup
    jsStatus.stop();
    jsStatus.setVisible(false);


    stationParameters = findParametersFromSpace();  //check for new params
    if(stationParameters==null)
    { stationParameters= findParametersFromDisk();  //get from disk if no new params
    }
    else
     newParams=true;               //we need to save the new params

    if(stationParameters==null)
    { System.out.println("Params are null");
      mbe.setVisible(true);       // the major buku error, no params at all

    }
    else
    {//check to see if id is included in the params
     Integer id= (Integer)stationParameters.getGuiParameters().getParameter("exhibitIDCode");
     if(id==null) //check js for it
     { if((this.stationIDCode=this.getStationIDFromSpaces(this.stationName))!=-1)
       { stationParameters.getGuiParameters().addToParameters("exhibitIDCode",new Integer(this.stationIDCode));
         System.out.print("found id");
         newParams=true;
       }
       else
       { System.out.println("No ID:");
         mbe.setVisible(true);
       }
     }
     else
     { this.stationIDCode=(short)id.intValue();
       System.out.println("id already in params");
     }
     if(newParams)                         //added the station ID automatically
       saveParams(stationParameters);
    }

    try {
      myDevice = (Device) (Class.forName(stationParameters.getDeviceName())).newInstance();
      myDeviceProxy = (DeviceProxy) (Class.forName(stationParameters.getDeviceProxyName())).newInstance();
      myDeviceProxy.setDeviceInterface(myDevice);           //do some error checks here
      stationValueMap = (ValueMap) (Class.forName(stationParameters.getValueMapName())).newInstance();

      Integer pdeviceActivated=(Integer)stationParameters.getGuiParameters().getParameter("deviceActivated");

      if(pdeviceActivated!=null)
      { MeasureMeGlobal.deviceActivated=(pdeviceActivated.intValue()==1);
        System.out.println("DeviceActivated: " + MeasureMeGlobal.deviceActivated);
      }
      else
        System.out.println("DeviceActivated: null");

    }
    catch(Exception e)
    { e.printStackTrace();
      DeviceErrorFrame def=new DeviceErrorFrame();
      System.exit(0);
    }
    /**
     *  ask the database for a new set of age data for this station
     */
    DatabaseQueryStatus dbStatus=new DatabaseQueryStatus();
    dbStatus.setVisible(true);
    RequestStationData rsd=new RequestStationData(this.stationName,this.javaSpace);

    try{
     this.stationAgeScores= rsd.getStationData(120000); // a total timeout of 2 min
     this.saveOldStationData(this.stationAgeScores);    //keep a local copy
     dbStatus.stop();
     dbStatus.setVisible(false);
     }
     catch(Exception e)
     { e.printStackTrace(); //need to try to recover
     }
     //was the lookup successful???
     if(this.stationAgeScores==null)
     { dbStatus.stop();
       this.stationAgeScores= this.findOldStationDataFromDisk();
       if(this.stationAgeScores!=null)
        dbStatus.setError("Warning, Using old station data.");
       else
       {  dbStatus.setError("Error, No station/age values.");
          try{
           Thread.sleep(60000);
           }
          catch(Exception e)
          {}
          System.exit(0);
       }
     }

    // and finally, get the id codes for country, county
    readCodesFromSpace();
    if((this.countryCodes==null) || (this.countyCodes==null) || (this.countryCodes.size()==0) || (this.countyCodes.size()==0) )
    { System.out.println("Codes null");
      if(!getCodesFromDisk())
      { //do some error code
      }
    }
    else
     saveCodestoDisk(this.countryCodes,this.countyCodes);

    //create the gui
    mainGui = new MeasureMeFrame(stationParameters,avatarInf);
    //create the object that rolls the client through demographic collection
    demDecision=new DemographicDecisions(mainGui,avatarInf,this.countryCodes,this.countyCodes);
    System.out.println("Station ID: "+this.stationIDCode);
    this.eam=new ExhibitAvailableManager(this.javaSpace,(short)(this.stationIDCode));

    //create the new exhibit available collection
    System.out.println("adding tasks");
    this.clAdmin=new ClientAdministration(this.mainGui,this);
    this.clAdmin.addAdministrationTask(new ShutDownTask());
    this.mainGui.addAdminRequestListener(this.clAdmin);
    this.mainGui.setDeviceProxy(this.myDeviceProxy);

    /** Hack */
    if(this.stationName.equals("Reaction Time"))
    { ((ReactionTimeDeviceProxy)this.myDeviceProxy).setMode("Chase");

    }
    if (stationName.equals("Jump")) {
      ((JumpDeviceProxy) myDeviceProxy).setChaseMode();
    }

    //Validate frames that have preset sizes
    //Pack frames that have useful preferred size info, e.g. from their layout
    if (packFrame)
      mainGui.pack();
    else
      mainGui.validate();
    //Center the window
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = mainGui.getSize();
    if (frameSize.height > screenSize.height)
      frameSize.height = screenSize.height;
    if (frameSize.width > screenSize.width)
      frameSize.width = screenSize.width;
    mainGui.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
    mainGui.setVisible(true);



    mainGui.addclientEntersStationListener(this);    //add all our event listeners
    mainGui.addClientLeavesStationListener(this);    //for client events
    mainGui.addClientEnteredDemographicsListener(this);
    mainGui.addClientStartsMeasurementListener(this);
    mainGui.addClientEndsStationListener(this);
    mainGui.addClientRestartsStationListener(this);
    String scanCode = MeasureMeGlobal.scanCode;
    if (scanCode != null)
    //start a new thread for the client
    try{
      eThread=new Thread(new ExhibitorThread(scanCode));
      eThread.setName("Visitor"+(String) scanCode);
      eThread.start();
    } catch(Exception e2) {
      System.out.println("Scan code " + scanCode + " failed to create an Exhibitor thread");
    }
  }

  //Main method
  public static void main(String[] args) {
    MeasureMeGlobal.init();
    try  {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch(Exception e) {
    }


    new MeasureMeClient();

  }

  /**
   * PURPOSE: A client entered the station.  Signaled when keyCodeHandler
   * receives a valid user bar code.
   */
  public void dataChanged(ClientEntersStationEvent e) {
    System.out.println("ClientEntersStation begin");
/*
    try {
      if (eThread.isAlive()) {
        System.out.println("***************************EXHIBITOR THREAD IS STILL ALIVE!*********************************");
      }
    }
    catch (NullPointerException e1) {}
*/
    // prevent user from losing his tuple if null code not scanned between
    // two very quick consecutive logins (null code didn't get scanned)
    try {
      if (eThread.isAlive()) {
        // visitor did not log out
        System.out.println("Visitor did not log off.  Ignoring request to log in.");
        return;
      }
    }
    catch (NullPointerException e1) {}

    System.out.println("Client entered: "+e.getSource());

    // if hang time, need to enable display of weight popup
    if (stationName.equals("Hang Time")) {
      //mainGui.htPopup.preventDisplay(false);
    }

    //start a new thread for the client
    try{
      eThread=new Thread(new ExhibitorThread((String) e.getSource()));
      eThread.setName("Visitor"+(String) e.getSource());
      eThread.start();
    }
    catch(Exception e2) {
      e2.printStackTrace();
    }
    System.out.println("ClientEntersStation end");
  } // dataChanged(ClientEntersStationEvent)

  /**
   * Purpose: signals that a client has left the station, perhaps abrubtly.
   * This event is fired when keyCodeHandler sees the NULL bar code. (0000).
   */
  public void dataChanged(ClientLeavesStationEvent e) {
    System.out.println("ClientLeavesStation begin");
    System.out.flush();
    if (eThread == null) {
      System.out.println("ClientLeavesStation end");
      System.out.flush();
      return;
    }
    else {
      if (!eThread.isAlive()) {
        System.out.println("ClientLeavesStation end");
        System.out.flush();
        return;
      }
    }

    // set flag variable
    MeasureMeGlobal.clientLeft=true;
    // we need to notify objects waiting on this object

    //System.out.println("setting demDecision to clientLeft");
    //System.out.flush();
    // return from demographic decision if in demographic collection
    //demDecision.clientLeft();

    //System.out.println("interrupting exhibitor thread");
    //System.out.flush();

    // we need to interrupt exhibitor thread waiting on some other object (not this object)
    //try {eThread.interrupt();} catch (NullPointerException e1) {}


    // this will make measure() or waitForDeviceMeasure() return quickly
    System.out.println("Calling stopMeasure");
    this.myDeviceProxy.stopMeasure();


    // raise priority so that it hurries up and finishes
    //try {eThread.setPriority(eThread.getPriority()+1);} catch (NullPointerException e1) {}

    // print out active threads
/*    ThreadGroup threadGroup = Thread.currentThread().getThreadGroup();
    Thread[] threads = new Thread[threadGroup.activeCount() + 5];
    int threadCount = threadGroup.enumerate(threads);
    for (int k = 0; k < threadCount; k++) {
      System.out.println(k + ":  " + threads[k].toString());
    }
*/
    System.out.println("WAITING ON EXHIBITOR THREAD TO DIE...");
    try {
      eThread.join();
    }
    catch (InterruptedException e1) {}

/*    synchronized(this) {
      try {
        while (eThread.isAlive()) {
          System.out.println("waiting on exhibitor thread to die...");
          wait(MeasureMeGlobal.WAIT_TIME);  // we need to give up the monitor for MeasureMeClient so that
                      // other synchronized methods can finish
        }
      }
      catch (InterruptedException e1) {}
      catch (NullPointerException e2) {}
    }
*/
    System.out.println("exhibitor thread is dead");
    System.out.flush();

    // print out active threads
/*    threads = new Thread[threadGroup.activeCount() + 5];
    threadCount = threadGroup.enumerate(threads);
    for (int k = 0; k < threadCount; k++) {
      System.out.println(k + ":  " + threads[k].toString());
    }
*/

    // put tuple back in space
    try {
      System.out.println("writing exhibitor back to space");
      System.out.flush();
      resendClient();
    }
    catch(Exception e1) {
      e1.printStackTrace();
    }

    ExhibitScoreInfo esi = this.currentExhibitor.getScoreInfo(this.stationIDCode);
    // score is exhibitor's best score during this login
    short score = new Short(esi.getScore()).shortValue();

    // if user actually made it to the measurement
    if (esi.getScore() != -1) {
      System.out.println("client leaving; saving best score of: "+esi.getScore());
      addToLocalDataset(new Short(this.currentExhibitor.getAge()), score);
    }

    System.out.println("We finished updating the local dataset!");
    esi.setLastVisitTime(new java.util.Date());


    // if hang time, do not display weight popup when curve finished drawing
    if (stationName.equals("Hang Time")) {
      //mainGui.htPopup.preventDisplay(true);
    }
    // reset to intro screen
    mainGui.flipToMainIntro();
     if(this.stationName.equals("Reaction Time")) {
      System.out.println("dataChanged(Leaves): (reaction time) setting chase sequence");
      ((ReactionTimeDeviceProxy)this.myDeviceProxy).setMode("Chase");
    }

    if (stationName.equals("Jump")) {
      ((JumpDeviceProxy) myDeviceProxy).setChaseMode();
    }

    currentExhibitor = null;

    System.out.println("ClientLeavesStation end");
    System.out.flush();
  } // dataChanged(ClientLeavesStationEvent)

  /**
   * PURPOSE: signifies the client has entered the demographic data
   */
  public void dataChanged(ClientEnteredDemographicsEvent e) {
    System.out.println("ClientEnteredDemographics begin");
    mainGui.flipToTeaching();
    System.out.println("ClientEnteredDemographics end");
  } // dataChanged(ClientEnteredDemographicsEvent)

  /**
   * The client has signaled that they wish to start collecting data.  This
   * method is only called when deviceActived == false.  i.e. this event
   * is fired when the user clicks the Try It button.
   */
  public void dataChanged(ClientStartsMeasurementEvent e) {
    new Thread() {
      public void run() {
    System.out.println("ClientStartsMeasurement begin");

    if (!MeasureMeGlobal.clientLeft) {
      mainGui.flipToCollectData();
    }

    if (!MeasureMeGlobal.clientLeft) {
      if (stationName.equals("Jump")) {
        System.out.println("flipToJumpCountdown");
        mainGui.flipToJumpCountdown();
      }
    }

    Object o = null;
    try {
      if (!MeasureMeGlobal.clientLeft) {
        o=myDeviceProxy.measure(); //our proxy returns measurement
      }
    }
    catch (InterruptedException e2) {
      e2.printStackTrace();
    }
    catch (DeviceFailureException e3) {
      e3.printStackTrace();
    }
      //addToLocalDataset(new Short(this.currentExhibitor.getAge()), stationValueMap.getInternalValue(o).shortValue());

      if (!MeasureMeGlobal.clientLeft) {
        if (stationName.equals("Jump")) {
          if (((Short)o).shortValue() == -1) {
            System.out.println("flipToRunningJump");
            mainGui.flipToRunningJump();
            // ClientRestartsStationEvent will be fired by runningJumpPanel
            return;
          }
        }
      }
      if (!MeasureMeGlobal.clientLeft) {
        addToCurrentUserScore(stationValueMap.getInternalValue(o));
      }

      if (!MeasureMeGlobal.clientLeft) {
        mainGui.showResults((Short[])stationAgeScores.get(new Short(currentExhibitor.getAge())),stationValueMap.getInternalValue(o).shortValue(),currentExhibitor.getAge());
      }
      //mainGui.showResults();
      //mainGui.showResults(o);

    System.out.println("ClientStartsMeasurement end");
      }
    }.start();
  } // dataChanged(ClientStartsMeasurementEvent)

  /**
   * PURPOSE: add the data collected to the local dataset
   */
  protected void addToLocalDataset(Short clientAge,short score)
  { System.out.println("Adding to local data, age:"+clientAge+" score: "+score);
    System.out.println("Size of stationAgeScores before: "+this.stationAgeScores.size());
    Short[] temp=(Short[])this.stationAgeScores.get(clientAge);
    if(score<=255)
    { if(temp!=null)
      { this.stationAgeScores.remove(clientAge);
        temp[score]=new Short((short)(temp[score].shortValue()+1));
        System.out.println("Size of dataset: "+temp.length);
        this.stationAgeScores.put(clientAge,temp); //update the local score
        System.out.println("Size of stationAgeScores after: "+this.stationAgeScores.size());
      }
    }
  }

  /**
   * PURPOSE: Add a users score to the local dataset;
   */
  protected void addToCurrentUserScore(Short score)
  {
    // only keep the best score from this login
    // esi is the "previous score" object
    ExhibitScoreInfo esi = this.currentExhibitor.getScoreInfo(this.stationIDCode);
    System.out.println("Previous score: "+esi.getScore());
    System.out.println("Current score: "+score);
    if (esi.getScore() == -1) {
      esi.setScore(score.shortValue());
    }
    else {
      if (!stationName.equals("Reaction Time")) {
        if (score.shortValue() > esi.getScore()) {
          System.out.println("Current score better than previous...setting new score...");
          esi.setScore(score.shortValue());
        }
      }
      else {
        if (score.shortValue() < esi.getScore()) {
          System.out.println("Current score better than previous...setting new score...");
          esi.setScore(score.shortValue());
        }
      }
    }
    System.out.println("Current exhibitor's best score: "+esi.getScore());
/*
  ExhibitScoreInfo esi=this.currentExhibitor.getScoreInfo(this.stationIDCode);
    esi.setScore(score.shortValue()); //updates score and times visited
*/
  }


  /**
   * PURPOSE: find the java spaces server
   */
  protected void lookup() {
    javaSpace=MMSpaceAccessor.getSpace();
  }


  /**
   * PURPOSE: Read a text file for config info
   * the name of the station is the first line, name of device, on the second then
   * the name of the proxy for the device on the third
   */
  public void getTextFile()
  { File f=new File(stationSetup);
    try{
     FileInputStream fis=new FileInputStream(f);
     BufferedInputStream bis=new BufferedInputStream(fis);
     AsciiInputStream ais=new AsciiInputStream(bis);
     this.stationName=ais.readLine();

     // save station name for later uses
     MeasureMeGlobal.stationName = this.stationName;

     System.out.println(stationName);
     ais.close();
     }
    catch(Exception e)
    { e.printStackTrace();
    }
  }

  /**
   *  PURPOSE: find the parameters from the disk
   */
  protected StationParameters findParametersFromDisk()
  { File f=new File(paramFile);
    StationParameters p=null;
    try{
     FileInputStream fis=new FileInputStream(f);
     ObjectInputStream ois=new ObjectInputStream(fis);
     System.out.println("reading param file");
     p=(StationParameters)ois.readObject();
     System.out.println("done reading param file");
     fis.close();
     return p;
     }
    catch(Exception e)
    { e.printStackTrace();

    }
   return null;
  }

  /**
   *  PURPOSE: find the parameters from the disk
   */
  protected Hashtable findOldStationDataFromDisk()
  { File f=new File(this.oldStationDataFile);
    Hashtable rh=null;
    try{
     FileInputStream fis=new FileInputStream(f);
     ObjectInputStream ois=new ObjectInputStream(fis);
     rh=(Hashtable)ois.readObject();
     fis.close();
     return rh;
     }
    catch(Exception e)
    { e.printStackTrace();

    }
   return null;
  }

  /**
   * PURPOSE: retrieve station paramters from javaspace
   */
  protected StationParameters findParametersFromSpace()
  { StationParameters template=new StationParameters();
    StationParameters p=null;
    template.stationName=stationName;
    try{ p=(StationParameters)javaSpace.takeIfExists(template,null,Long.MAX_VALUE);
         return p;
         }
      catch(Exception e2)
      {
        e2.getMessage();
        e2.printStackTrace();
      }
   return null;
  }

  /**
   *  PURPOSE: save the parameters to disk
   */
  protected void saveParams(StationParameters sp)
  {
    if(sp!=null)
    {  File f=new File(paramFile);
       try
        { FileOutputStream fos=new FileOutputStream(f);
          ObjectOutputStream oos=new ObjectOutputStream(fos);
          oos.writeObject(sp);
          fos.close();
        }
        catch(Exception e2)
        { e2.printStackTrace();
        }
     }
  }

  protected void saveOldStationData(Hashtable oldData)
  { if(oldData!=null)
    {  File f=new File(this.oldStationDataFile);
       try
        { FileOutputStream fos=new FileOutputStream(f);
          ObjectOutputStream oos=new ObjectOutputStream(fos);
          oos.writeObject(oldData);
          fos.close();
        }
        catch(Exception e2)
        { e2.printStackTrace();
        }
     }
  }

/*  protected DeviceProxy getDeviceProxy(String proxyName)
  { Class proxyClass=null;
   try{
      proxyClass=Class.forName(proxyName);
     }
     catch(Exception e)
     { e.printStackTrace();
       //statusLabel.setText("invalid class name, use fully qualified class name");
       return null;
     }
     try{
      DeviceProxy p=(DeviceProxy)proxyClass.newInstance();

      return p;
        }
     catch(Exception e2)
     { e2.printStackTrace();

       return null;
     }
  // return null;

  }
*/
/*  protected Device loadDevice(String deviceName)
  { Class devClass=null;
   try{
      devClass=Class.forName(deviceName);
     }
     catch(Exception e)
     { e.printStackTrace();
       //statusLabel.setText("invalid class name, use fully qualified class name");
       return null;
     }
     try{
      Device d=(Device)devClass.newInstance();
      return d;
        }
     catch(Exception e2)
     { e2.printStackTrace();

       return null;
     }

  }
*/
  /**
   * PURPOSE: check java spaces for this stations ID
   */
  protected short getStationIDFromSpaces(String stationName)
  {  QueryResults template= new QueryResults();
     template.setReplyToID("StationIDs");  //find
     QueryResults result=null;
     ResultHolder rh=null;
     if( this.javaSpace!=null)
     {  try{ result=(QueryResults)javaSpace.read(template,null,10000);
              if (result == null) {
                return -1;
              }
             rh=result.getResults();
             Vector v=rh.getColumn(1);
             int pos=v.indexOf(this.stationName);
             if(pos!=-1)
             { Vector v2=rh.getRow(pos+1);
               Integer val=(Integer)v2.elementAt(1);
               System.out.println("Station id is: "+val);
               return (short)val.intValue();
             }
            }
         catch(Exception e2)
         { e2.printStackTrace();
         }
      }
      return -1;

  }

/**
 * PURPOSE: write country codes to disk
 */
  protected void saveCodestoDisk(Hashtable country,Hashtable county)
  {
      if((country!=null)&&(county!=null))
      { File f=new File(codeFile);
        try
        { FileOutputStream fos=new FileOutputStream(f);
          ObjectOutputStream oos=new ObjectOutputStream(fos);
          oos.writeObject(country);
          oos.writeObject(county);
          fos.close();
        }
        catch(Exception e2)
        { e2.printStackTrace();
        }
     }

  }

  /**
   * PURPOSE: Read the country,county codes from disk.
   */
  protected boolean getCodesFromDisk()
  { File f=new File(codeFile);
    if(!f.exists())
     return false;
    //StationParameters p=null;
    try{
     FileInputStream fis=new FileInputStream(f);
     ObjectInputStream ois=new ObjectInputStream(fis);
     this.countryCodes=(Hashtable)ois.readObject();
     this.countyCodes=(Hashtable)ois.readObject();
     fis.close();
     return true;

     }
    catch(Exception e)
    { e.printStackTrace();

    }
    return false;
  }

  /**
   * PURPOSE: read the country,county codes out of the javaspace
   */
  protected void readCodesFromSpace()
  {  QueryResults template= new QueryResults();
     template.setReplyToID("CountryCodes");  //find
     QueryResults result=null;
     ResultHolder rh=null;
     if( this.javaSpace!=null)
     {  try{ result=(QueryResults)javaSpace.read(template,null,10000);
        if (result == null) {return;}
             rh=result.getResults();
             Vector v1=rh.getColumn(1);
             Vector v2=rh.getColumn(2);
             this.countryCodes=new Hashtable();
             for(int i=0;i<v2.size();i++)
             { countryCodes.put(v1.elementAt(i),v2.elementAt(i) );
               //System.out.println(v1.elementAt(i)+" "+v2.elementAt(i)+" "+v2.elementAt(i).getClass());
             }

             template.setReplyToID("CountyCodes");  //find
             result=(QueryResults)javaSpace.read(template,null,10000);
             rh=result.getResults();
              v1=rh.getColumn(1);
              v2=rh.getColumn(2);
             this.countyCodes=new Hashtable();
             for(int i=0;i<v2.size();i++)
             { countyCodes.put(v1.elementAt(i),v2.elementAt(i) );
               //System.out.println(v1.elementAt(i)+" "+v2.elementAt(i));
             }
            }
         catch(Exception e2)
         { e2.printStackTrace();
         }
      }



  }

  /**
   * PURPOSE: This event is called when user clicks the Yes button
   * when prompted to try again.
   */
  public void dataChanged(ClientRestartsStationEvent e) {
    System.out.println("ClientRestartsStation begin");
    this.restartRequest=true;
    System.out.println("ClientRestartsStation end");
  } // dataChanged(ClientRestartsStationEvent)

  /**
   * PURPOSE: The client has signaled that he wishes to quit the station,
   * so show him the ending screen.  Fired when user clicks the No button,
   * when prompted to try again.
   */
  public void dataChanged(ClientEndsStationEvent e) {
    System.out.println("ClientEndsStation begin");
    this.mainGui.flipToEndPanel(this.currentExhibitor,this.eam.getLastActiveStations());
    System.out.println("ClientEndsStation end");
  } // dataChanged(ClientEndsStationEvent)

  public DeviceProxy getDeviceProxy() {
    return this.myDeviceProxy;
  }

  /**
   * Write Exhibitor back to space.
   */
  protected void resendClient() throws Exception {
    ExhibitorWrapper wrapper=new ExhibitorWrapper(currentExhibitor);
    javaSpace.write(wrapper,null,Lease.FOREVER);
  }


} // MeasureMeClient