package edu.ucf.cs.dml.simnets.measureme.dbase;

/**
 * <p>Title: MeasureMe Database project</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) Dean Reed</p>
 * <p>Company: IST-DML</p>
 * @author Mathew Lowery
 * Modified: 2002-Oct-08 (turned off night time listener)
 * @version 1.0
 */

import edu.ucf.cs.dml.simnets.measureme.server.*;
import edu.ucf.cs.dml.simnets.measureme.MeasureMeGlobal;
import java.beans.*;
import net.jini.space.*;
import java.util.*;

/**
 * Command-line version of MeasureMeDBQueryFrame.
 */
public class DBProxy implements PropertyChangeListener,
                                NightTimeUpdateFailedListener,
                                NightTimeUpdateSuccessListener {
  protected MeasureMeSQLQueryObserver queryObserver = null;
  protected JavaSpace space = null;
  protected NightTimeExhibitorUpdates nightTimeUpdates = null;
  protected StationAvailableServer stationAvailableServer = null;
  protected int nightTimeRuns = 0;

  public DBProxy() throws Exception {
    space = MMSpaceAccessor.getSpace();

    // listen for queries processed
    queryObserver = new MeasureMeSQLQueryObserver(space);
    queryObserver.addPropertyChangeListener(this);

    // listen for station availability info
    stationAvailableServer = new StationAvailableServer();
    stationAvailableServer.addPropertyChangeListener(this);

    // listen for night time updates
    // nightTimeUpdates = new NightTimeExhibitorUpdates(space, 20);
    // nightTimeUpdates.addNightTimeUpdateFailedListener(this);
    // nightTimeUpdates.addNightTimeUpdateSuccessListener(this);
  }

  public static void main(String[] args) throws Exception {
    MeasureMeGlobal.init();
    System.out.println("********* Restart Jini stuff before running this " +
                       "to ensure a clean space *********");
    new DBProxy();
  }

  public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
    // event fired from MeasureMeSQLQueryObserver
    if (propertyChangeEvent.getPropertyName().equals("numQueriesProcessed")) {
      System.out.println("EVENT=numQueriesProcessed: " +
                         propertyChangeEvent.getNewValue());
    }

    // event fired from StationAvailableServer
    if (propertyChangeEvent.getPropertyName().equals("lastActiveStations")) {
      Vector stationsAvailable = new Vector();
      Enumeration enum = stationAvailableServer.getLastActiveStations().elements();
      ExhibitAvailableInfo info = null;
      while(enum.hasMoreElements()) {
        info = (ExhibitAvailableInfo) enum.nextElement();
        stationsAvailable.add(info.getStationID() + "=" + info.getStationName());
      }
      System.out.println("EVENT=lastActiveStations: " + stationsAvailable);
    }
  }

  /**
   * Method defined in NightTimeUpdateFailedListener
   * @param e
   */
  public void dataChanged(NightTimeUpdateFailedEvent e) {
    System.err.println("EVENT=NightTimeUpdateFailed: at least one " +
                       "tuple did not get updated in the database");
  }

  /**
   * Method defined in NightTimeUpdateSuccessListener
   * @param e
   */
  public void dataChanged(NightTimeUpdateSuccessEvent e) {
    System.out.println("EVENT=NightTimeUpdateSuccess: total runs = " +
                       ++nightTimeRuns);
  }
} // DBProxy