
/**
 * Title:        MeasureMe Database project<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Dean Reed<p>
 * Company:      IST-DML<p>
 *  CLASS NightTimeExhibitorUpdates: a class that monitors the local clock
 *   for a given hour and then removes all exhibitors from the javaspace
 *   and returns a database update given by the exhibitor back to the javaspace.
 * @author Dean Reed
 * @version 1.0
 */
package edu.ucf.cs.dml.simnets.measureme.dbase;

import edu.ucf.cs.dml.simnets.measureme.server.*;

import java.util.*;
import java.sql.*;
import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;
import edu.ucf.cs.dml.simnets.measureme.server.SpaceAccessor;

public class NightTimeExhibitorUpdates implements Runnable
{ protected JavaSpace space=null;
  private Integer hourToRun=null;
  private Integer minuteToRun=null;
  private transient Vector nightTimeUpdateFailedListeners;
  private transient Vector nightTimeUpdateSuccessListeners;
  protected Thread myRunThread=null;
  protected boolean alreadyRanToday=false;
  protected boolean run=true;




  /**
   * CONSTRUCTOR: constructs  a nighttimeexhibitorupdate based on
   *  the 24 hour clock. I.E. 20 = 8:00 pm
   */
  public NightTimeExhibitorUpdates(JavaSpace space, int hourToRun)
  { this.hourToRun=new Integer(hourToRun);
    this.minuteToRun=new Integer(0);
    this.space=space;
    this.myRunThread=new Thread(this);
    this.myRunThread.start();
  }

  public static void main(String[] args)
  { try{
     JavaSpace someSpace=MMSpaceAccessor.getSpace();
     NightTimeExhibitorUpdates nightTimeExhibitorUpdates1 = new NightTimeExhibitorUpdates(someSpace,20);
     System.out.println("NightTimeUpdates is now running independantly");
      }
    catch(Exception e)
    { e.printStackTrace();
    }

  }

  public boolean runUpdates()
  { System.out.println("running nightime update!");
    ExhibitorWrapper template =new ExhibitorWrapper();
    ExhibitorWrapper theExhibitorWrap = null;
    Exhibitor currEx=null;
    boolean exibitorsExist=true;
    QueryReplyRequest qrr=null;
    int numberOfClientUpdates=0;
     while(exibitorsExist)
      { try
        { theExhibitorWrap=(ExhibitorWrapper)space.takeIfExists(template,null,20000);
          if(theExhibitorWrap!=null)
          { currEx=theExhibitorWrap.getExhibitor();
            System.out.println("Removing exhibitor, and querying with: "+currEx.getQueryUpdateString());
            qrr=new QueryReplyRequest(currEx.getQueryUpdateString(),"NightTime",false);
            space.write(qrr,null,Lease.FOREVER);
            //this still puts a response back into the space, we can check them later
            numberOfClientUpdates++;
          }
          else
          {  exibitorsExist=false;
          }
        }
        catch(Exception e2)
        { e2.printStackTrace();
          exibitorsExist=false;
          return false;
        }
     }
     QueryResults template2=new QueryResults();
     template2.setReplyToID("NightTime");
     QueryResults qr=null;
     boolean gotResults=true;
     int i=0;
     while(gotResults && (i<numberOfClientUpdates))
     {  try{
    // Edit M. Lowery; 2002-Jul-30 changed takeIfExists to take since
    // takeIfExists does not wait for timeout before returning
          qr=(QueryResults)space.take(template2,null,30000);
          if(qr!=null)
          { i++;
            if(qr.getError()!=null)
             System.out.println("DBError report: "+qr.getError());
          }
          else
           gotResults=false;
          }
        catch(Exception e)
        { e.printStackTrace();
        }
     }
     if(i!=numberOfClientUpdates)
      // edit M. Lowery 2002-Jul-30
      // changed error printout to numberOfClientUpdates - i
      System.out.println("Error, did not recieve "+(numberOfClientUpdates-i)+ " updates");
     return true;
  }



  public void run()
  { System.out.println("nightime thread: start.");
    Calendar calendar =Calendar.getInstance();
    while(run)
    {  calendar.setTime(new java.util.Date());
      while((calendar.get(Calendar.HOUR_OF_DAY) < this.hourToRun.intValue()) && run )
      { System.out.println("Current Hour: "+ calendar.get(Calendar.HOUR_OF_DAY) +" waiting for :"+ this.hourToRun );
        try{
         Thread.sleep(100000); //sleep a hundred seconds
           }
        catch(Exception e1)
        {e1.printStackTrace();
        }
        calendar.setTime(new java.util.Date());
      }
      if(!run)
       return;
      while(alreadyRanToday && run)
      { try{
         Thread.sleep(100000); //reset self after midnight
           }
        catch(Exception e2)
        {e2.printStackTrace();
        }
        calendar.setTime(new java.util.Date());
        if(calendar.get(Calendar.HOUR_OF_DAY) <=1)
          this.alreadyRanToday=false;
      }
      if(!run)
       return;
      calendar.setTime(new java.util.Date());
      if(!(calendar.get(Calendar.HOUR_OF_DAY) < this.hourToRun.intValue()))
       if(!alreadyRanToday)
        if(this.runUpdates())
        { this.fireDataChanged(new NightTimeUpdateSuccessEvent(this));
          this.alreadyRanToday=true;
          //this.fifinalize();
        }
        else
        { //try to reync javaspace
           try{ this.space=MMSpaceAccessor.getSpace();
              }
           catch(Exception e)
           {e.printStackTrace();}
           if(this.runUpdates())
           { this.fireDataChanged(new NightTimeUpdateSuccessEvent(this));
             this.alreadyRanToday=true;
           }
           else
            this.fireDataChanged(new NightTimeUpdateFailedEvent(this));

        }//end else

     }//end while run
     System.out.println("nightime thread: start.");
  }

  public void setHourToRun(Integer newHourToRun)
  {
    hourToRun = newHourToRun;
  }

  public Integer getHourToRun()
  {
    return hourToRun;
  }

  public void setMinuteToRun(Integer newMinuteToRun)
  {
    minuteToRun = newMinuteToRun;
  }

  public Integer getMinuteToRun()
  {
    return minuteToRun;
  }

  public synchronized void removeNightTimeUpdateFailedListener(NightTimeUpdateFailedListener l)
  {
    if (nightTimeUpdateFailedListeners != null && nightTimeUpdateFailedListeners.contains(l))
    {
      Vector v = (Vector) nightTimeUpdateFailedListeners.clone();
      v.removeElement(l);
      nightTimeUpdateFailedListeners = v;
    }
  }

  public synchronized void addNightTimeUpdateFailedListener(NightTimeUpdateFailedListener l)
  {
    Vector v = nightTimeUpdateFailedListeners == null ? new Vector(2) : (Vector) nightTimeUpdateFailedListeners.clone();
    if (!v.contains(l))
    {
      v.addElement(l);
      nightTimeUpdateFailedListeners = v;
    }
  }

  protected void fireDataChanged(NightTimeUpdateFailedEvent e)
  {
    if (nightTimeUpdateFailedListeners != null)
    {
      Vector listeners = nightTimeUpdateFailedListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++)
      {
        ((NightTimeUpdateFailedListener) listeners.elementAt(i)).dataChanged(e);
      }
    }
  }

  public synchronized void removeNightTimeUpdateSuccessListener(NightTimeUpdateSuccessListener l)
  {
    if (nightTimeUpdateSuccessListeners != null && nightTimeUpdateSuccessListeners.contains(l))
    {
      Vector v = (Vector) nightTimeUpdateSuccessListeners.clone();
      v.removeElement(l);
      nightTimeUpdateSuccessListeners = v;
    }
  }

  public synchronized void addNightTimeUpdateSuccessListener(NightTimeUpdateSuccessListener l)
  {
    Vector v = nightTimeUpdateSuccessListeners == null ? new Vector(2) : (Vector) nightTimeUpdateSuccessListeners.clone();
    if (!v.contains(l))
    {
      v.addElement(l);
      nightTimeUpdateSuccessListeners = v;
    }
  }

  protected void fireDataChanged(NightTimeUpdateSuccessEvent e)
  {
    if (nightTimeUpdateSuccessListeners != null)
    {
      Vector listeners = nightTimeUpdateSuccessListeners;
      int count = listeners.size();
      for (int i = 0; i < count; i++)
      {
        ((NightTimeUpdateSuccessListener) listeners.elementAt(i)).dataChanged(e);
      }
    }
  }

  public void stopRunning()
  { this.run=false;

  }




}