package edu.ucf.cs.dml.simnets.measureme.devices;
import java.beans.*;
import edu.ucf.cs.dml.simnets.measureme.*;

/**
 * Title:        Devices for Measure Me
 * Description:  Device Wrapppers/Code for measure me.
 * Copyright:    Copyright (c) 1999
 * Company:      DML
 * @author Dean Reed
 * @version
 */

public class ArmSpanDeviceProxy extends DeviceProxy {
  protected Short value=new Short((short)-1);
  boolean newValueFromDevice=false;

  public ArmSpanDeviceProxy() {
    super();
    newValueFromDevice=false;
  }

  public ArmSpanDeviceProxy(DeviceInterface deviceInterface) {
    super(deviceInterface);
    newValueFromDevice=false;
  }

  /**
   * When data bytes come in, they are of the form
   * <command, left, right> where left and right will be in the
   * range from 0..127.  Left and right are indices in the array
   * of LEDs.
   */
  public void propertyChange(PropertyChangeEvent evt) {
    Short temp;
    Short left=null;
    Short right=null;
    System.out.println("propertyChange: event");
    try {
      String cmdValue = this.deviceInterface.readFromDevice();
      switch (Integer.parseInt(cmdValue)) {
        case 2:
          // consume following 2 bytes (meaningless)
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();

          // reset value
          value = new Short((short)0);

          System.out.println("got device ack");
          newValueFromDevice=true;
          //System.out.println("got data; notifying waiters");
          //this.notifyAll();
          break;
        case 17:
          left =new Short(this.deviceInterface.readFromDevice());
          right=new Short(this.deviceInterface.readFromDevice());
          temp = new Short((short)((128-left.shortValue())+(128-right.shortValue())));
          // if temp greater than value
          if (temp.compareTo(value) > 1) {
            value = temp;
          }
          newValueFromDevice=true;
          //System.out.println("got data; notifying waiters");
          //this.notifyAll();
          break;
        default: System.out.println("Unknown command byte: "+ cmdValue);
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();
          break;
      } // switch
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  } // propertyChange

  public void waitForDeviceMeasure() throws DeviceFailureException,
                                                         InterruptedException {
    super.waitForDeviceMeasure();
    this.newValueFromDevice = false;
    deviceInterface.start();
    while (!newValueFromDevice && !stopped) {
      System.out.println("ArmSpanDeviceProxy.waitForDeviceMeasure() waiting");
      synchronized(this) {wait(MeasureMeGlobal.WAIT_TIME);}
    }
    if (stopped) {
      deviceInterface.stop();
    }

/*
      if (e2 instanceof InterruptedException) {
        this.deviceInterface.stop();
        throw (InterruptedException) e2;
      }

    try {
      while(!newValueFromDevice) {
        wait();
      }
      this.newValueFromDevice = false;
    }
    catch(Exception e) {
      if (e instanceof InterruptedException)
        throw (InterruptedException) e;
      else
        e.printStackTrace();
    }
  */
  } // waitForDeviceMeasure

  public Object measure() throws DeviceFailureException,
                                              InterruptedException {
    deviceInterface.stop();
    return value;
  } // measure

} // ArmSpanDeviceProxy