package edu.ucf.cs.dml.simnets.measureme.devices;

import edu.ucf.cs.dml.simnets.measureme.client.ArmSpanValueMap;

/**
 * Note on ArmSpanSimulator's behavior:  If the value entered in the sim window
 * is less than the currently stored value in the device proxy object, the
 * device proxy will ignore it.  However, on the next "iteration" of the
 * station, the value will be the one entered during last "iteration."  There is
 * no problem with entering a value higher than the currently stored one.
 */
public class ArmSpanSimulator extends NDADeviceSimulator {
  private static final int MEASUREMENT_INDICATOR = 17;

  public ArmSpanSimulator() {
    super();
  }

  protected String compute(Object userValue) {
    float lengthInInches = ((Float) userValue).floatValue();
    float adjustedLengthInInches = lengthInInches - ArmSpanValueMap.LOWEST_MEASUREMENT;
    // number of obscured LEDs
    short leds;
    if (Math.round(adjustedLengthInInches / ArmSpanValueMap.DIST_BETWEEN_LEDS) > 255) {
      leds = 255;
    }
    else {
      leds = (short) Math.round(adjustedLengthInInches / ArmSpanValueMap.DIST_BETWEEN_LEDS);
    }
    int left;
    int right;
    // if even then both left and right are same; otherwise right is one more than left
    if ((leds % 2) == 0) {
      left = right = (int) (leds / 2);
    }
    else {
      left = (int) (leds / 2);
      right = left + 1;
    }

    // now adjust since leds are numbered 0 to 128 starting on ends and working
    //   in
    left = 128 - left;
    right = 128 - right;

    // create char array for easy 3 char String creation
    char[] commandAndDataArray = new char[3];
    commandAndDataArray[0] = MEASUREMENT_INDICATOR;
    System.out.println("Command byte: "+(short)commandAndDataArray[0]);
    commandAndDataArray[1] = (char) left;
    System.out.println("First data byte: "+(short)commandAndDataArray[1]);
    commandAndDataArray[2] = (char) right;
    System.out.println("Second data byte: "+(short)commandAndDataArray[2]);
    String commandAndDataString = new String(commandAndDataArray);
    return commandAndDataString;
  }

  protected String getDefaultValue() {
    return compute(new Float(70.0F));
  }
} // ArmSpanSimulator