
/**
 * Title:        Devices for Measure Me<p>
 * Description:  Device Wrapppers/Code for measure me.<p>
 * Copyright:    Copyright (c) 1999<p>
 * Company:      DML<p>
 * @author Dean Reed
 * @version
 */
package edu.ucf.cs.dml.simnets.measureme.devices;
import edu.ucf.cs.dml.simnets.measureme.client.BalanceValueMap;
import edu.ucf.cs.dml.simnets.measureme.*;

import java.beans.PropertyChangeEvent;

public class BalanceDeviceProxy extends DeviceProxy {
  boolean personBalancing=false;
  boolean deviceAcknowledged=false;
  //boolean deviceFinished=false;
  long startTime=0;

  public BalanceDeviceProxy() {
    super();
  }

  public void propertyChange(PropertyChangeEvent parm1) {
    try {
      System.out.println("Mat - propertyChange on BalanceDeviceProxy");
      String cmdValue = this.deviceInterface.readFromDevice();
      switch (Integer.parseInt(cmdValue)) {

        // acknowledge
        case 2: {
          System.out.println("Mat - Got acknowledge!");
          deviceAcknowledged = true;
          // consume the other two data bytes (meaningless)
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();
          //notifyAll();
          break;
        } // case 2

        // data
        case 5: {
          System.out.println("Mat - Got data!");
          if (deviceAcknowledged) {
            deviceAcknowledged = false;
            personBalancing = true;
            //notifyAll();     // notify this object so waitForDeviceMeasure() can exit
          }
          // get data bytes (2nd and 3rd bytes)
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();
          break;
        } // case 3

        // finished measuring
        case 6: {
          System.out.println("Mat - Got finished signal!**********************");
          personBalancing=false;
          // consume the other two data bytes (meaningless)
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();
          //notifyAll();     // notify this object so measure() can exit
          break;
        } // case 4

        // ignore all other command values
        default: {
          System.out.println("Mat - Unrecognized command value!  Ignoring...");
          // consume the other two data bytes (meaningless)
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();
          break;
        } // default

      } // switch
    }
    catch (Exception e) {
      if (e instanceof InterruptedException) {
        System.out.println("Mat - interrupted while Device.readFromDevice()");
      }
      else {
         e.printStackTrace();
      }
    }
  } // propertyChange

  public void waitForDeviceMeasure() throws DeviceFailureException, InterruptedException {
    super.waitForDeviceMeasure();
    personBalancing=false;
    this.deviceInterface.removeDevicePropertyChangeListener(this);
    this.deviceInterface.addDevicePropertyChangeListener(this);
    this.deviceInterface.start();
    while(!personBalancing && !stopped) {   //wait for a device event
      System.out.println("BalanceDeviceProxy.waitForDeviceMeasure() waiting");
      synchronized(this) {wait(MeasureMeGlobal.WAIT_TIME);}
    }
    if (stopped) {
      deviceInterface.stop();
    }
    this.startTime=System.currentTimeMillis();
  }

  public Object measure() throws DeviceFailureException, InterruptedException {
    //this.deviceFinished=false;
    System.out.println("Waiting for end signal from the device");
    while(personBalancing && !stopped) {
      System.out.println("BalanceDeviceProxy.measure() waiting");
      synchronized(this) {wait(MeasureMeGlobal.WAIT_TIME);}
    }
    long endTime=System.currentTimeMillis();
    deviceInterface.stop();
    return new Long(endTime-this.startTime);
  } // measure


  public static void main(String[] args)
  {
    BalanceDeviceProxy vBalanceDeviceProxy = null;
    try
    {
      //UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch(Exception e)
    {
    }
   // new LoadCellTest();
   try{

   vBalanceDeviceProxy = new BalanceDeviceProxy();
   BalanceDevice htd=new BalanceDevice();
   //htd.initialize();
   vBalanceDeviceProxy.setDeviceInterface(htd);
   htd.removeDevicePropertyChangeListener(vBalanceDeviceProxy);
   System.out.println("Waiting for device use");
   vBalanceDeviceProxy.waitForDeviceMeasure();
   System.out.println("Done: Waiting for device use");
   Long time=(Long)vBalanceDeviceProxy.measure();
   //BalanceValue val=(BalanceValue)vBalanceDeviceProxy.measure();
   System.out.println("Time was: "+time);
   BalanceValueMap bvm=new BalanceValueMap();
   System.out.println("That value maps to: "+bvm.getInternalValue(time));

   //vBalanceDeviceProxy.st


   }catch (Exception e){
    System.out.println("error in main:");
    e.printStackTrace();}
   }


} // BalanceDeviceProxy