package edu.ucf.cs.dml.simnets.measureme.devices;

public class BalanceSimulator extends OSCDeviceSimulator {
  private long timeToBalance;
  // next two vars signal start and stop of balance action
  private static final int START_INDICATOR = 5;
  private static final int STOP_INDICATOR = 6;

  public BalanceSimulator() {
    super();
    timeToBalance = 1500;
  }

  public void guiEvent(SimulatorGUIEvent e) {
    // object at this point is a valid float of time in seconds; convert to ms
    float timeInSeconds = ((Float) e.getData()).floatValue();
    long timeInMilliseconds = (long) (timeInSeconds * 1000);
    timeToBalance =  timeInMilliseconds;
  }

  /**
   * The only time this is used is to send login codes.  For the purpose of the
   * simulator, the actual code is not used.  The call itself represents the
   * login.  Note that it takes 3 calls to this function to send the entire login
   * code so after the third call we say "received..."
   * @param data login code
   */
  public void write(int data) {
    if ((++loginCodeByteCounter % 3) == 0) {
      // start thread here to allow app to continue processing device signals
      new Thread(new Runnable() {
        public void run() {
          simGUI.appendStatus("Sim: Got login code...");
          loginCodeByteCounter = 0;
          char[] commandAndDataArray = new char[3];
          String commandAndDataString;
          commandAndDataString = new String(OSCDeviceSimulator.ACK_SIGNAL);
          simGUI.appendStatus("Sim: Sending ack signal...");
          fireSimulatorEvent(new SimulatorEvent(this, commandAndDataString));

          // pause for one sec just for reading screen
          try {
            Thread.currentThread().sleep(1000);
          } catch (InterruptedException e) { }


          // create char array for easy 3 char String creation
          commandAndDataArray[0] = (char) START_INDICATOR;
          commandAndDataString = new String(commandAndDataArray);
          simGUI.appendStatus("Sim: Sending start signal...");
          fireSimulatorEvent(new SimulatorEvent(this, commandAndDataString));
          try {
            Thread.currentThread().sleep(timeToBalance);
          } catch (InterruptedException e) { }
          commandAndDataArray[0] = (char) STOP_INDICATOR;
          commandAndDataString = new String(commandAndDataArray);
          simGUI.appendStatus("Sim: Sending stop signal...");
          fireSimulatorEvent(new SimulatorEvent(this, commandAndDataString));
        } // run
      }).start(); // Thread
    } // if
  }
}