package edu.ucf.cs.dml.simnets.measureme.devices;
import java.util.*;
import java.beans.*;

/**
 * INTERFACE DeviceInterface: The abstract layer that the MeasureMe client
 *  will use to communicate with the various physical devices.
 *  @author Dean Reed 6/02/00
 */
public interface DeviceInterface
{ /**
   * PURPOSE: Initialize the device that implements this interface. Initialize
   *  in this case means to reset the device, clear any old data and prepare for a fresh
   *  "session".
   */
  public void initialize()throws DeviceFailureException;

  /**
   * PURPOSE: Add a listener to this Device.
   */
  public void start()throws DeviceFailureException;

  /**
  * PURPOSE: open appropriate communication channels for device.
  */
  public void stop()throws DeviceFailureException;

  /**
  * PURPOSE: close appropriate communication channels for device.
  */
  public void addDevicePropertyChangeListener(DevicePropertyChangeListener l);

  /**
   * PURPOSE: remove a listener from this device
   */
  public void removeDevicePropertyChangeListener(DevicePropertyChangeListener l);

  /**
   * PURPOSE: A non blocking read from the device.
   * @throws NoDataReadyException when no data is available
   */
  //public String readFromDeviceImmediately()throws NoDataReadyException;

  /**
   * PURPOSE: A blocking read from the device.
   * @throws InterruptedExcpetion when the thread is interupted
   */
  public String readFromDevice();

  /**
   * PURPOSE: write the data string directly to the device.
   * @throws DeviceDataException if deviceDataString is invalid
   */
  public void writeToDevice(String deviceDataString) throws DeviceDataException;

  /**
   * PURPOSE: write a data string to the device with the given protocol
   * @throws DeviceDataException  if deviceDataString is invalid
   * @throws UnknownProtocolException if the device does not know the protocol String
   * @param deviceDataString - the data to write to the device in String format
   * @param protocolString the protocol command to use
   */
  public void writeToDevice(String deviceDataString,String protocolString) throws DeviceDataException,
                                                                                  UnknownProtocolException;


  /**
   * PURPOSE: return the valid protocol commands that this device can handle
   */
  public java.util.Enumeration getProtocolCmds();



}