
/**
 * CLASS DeviceProxy: contains the logic on how a device should be used.
 * @author Dean Reed
 * @version
 */
package edu.ucf.cs.dml.simnets.measureme.devices;

import java.beans.PropertyChangeEvent;

public abstract class DeviceProxy implements DevicePropertyChangeListener, java.io.Serializable
{ protected  DeviceInterface deviceInterface=null;
  protected boolean stopped;

  public DeviceProxy(DeviceInterface deviceInterface) {
    super();
    this.deviceInterface=deviceInterface;
    deviceInterface.addDevicePropertyChangeListener(this);
    try{
     this.deviceInterface.initialize();
     }
    catch(Exception e)
    { e.printStackTrace();
    }
  }

  public DeviceProxy() {
    super();
  }

  public void setDeviceInterface(DeviceInterface deviceInterface) throws DeviceFailureException
  {  if(this.deviceInterface!=null)
      this.deviceInterface.removeDevicePropertyChangeListener(this);

     this.deviceInterface=deviceInterface;
     this.deviceInterface.addDevicePropertyChangeListener(this);
     this.deviceInterface.initialize();

  }

  abstract public void propertyChange(PropertyChangeEvent evt);

  /**
   *  PURPOSE: do whatever measurement is neccessary and return the result
   */

  abstract public Object measure() throws DeviceFailureException, InterruptedException;
  /**
   * PURPOSE: if this device actually starts the datacollection via
   *  device signal, then this method blocks until the device detects the start,
   *  then the subsequent call to measure will have all pertinent values
   */

  public void waitForDeviceMeasure() throws DeviceFailureException, InterruptedException {
    // subclasses should call super.waitForDeviceMeasure() so
    // that this line is executed
    stopped = false;
  }

  /**
   * Added by M. Lowery 2001-05-25; Needed cleaner way of unblocking measure() and
   * waitForDeviceMeasure() methods instead of Thread.interrupt().
   */
  public void stopMeasure() {
    stopped = true;
  }
}