package edu.ucf.cs.dml.simnets.measureme.devices;

import java.util.Vector;

public abstract class DeviceSimulator implements SimulatorGUIListener {
  protected Vector listeners = new Vector();
  protected DeviceSimulatorGUI simGUI;

  /**
   * Represents a break signal in the simulator.  This string must not match any
   * other understood signal of the device.
   */
  public static final String BREAK_STRING = "...break!";

  public DeviceSimulator() {
    // create, add this as listener, and show the associated GUI object
    Class thisClass = this.getClass();
    String thisClassName = thisClass.getName();
    String guiClassName = thisClassName + "GUI";
    try {
      simGUI = (DeviceSimulatorGUI) Class.forName(guiClassName).newInstance();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    simGUI.addSimulatorGUIListener(this);
    simGUI.setVisible(true);
  }

  /**
   * Fires event <code>e</code>.
   * @param e the event
   */
  protected void fireSimulatorEvent(SimulatorEvent e) {
    for (int i = 0; i < listeners.size(); i++) {
      ((SimulatorListener) listeners.elementAt(i)).simulatorEvent(e);
    }
  }

  /**
   * Adds a listener.
   * @param l the listener
   */
  public void addSimulatorListener(SimulatorListener l) {
    if (!listeners.contains(l)) {
      listeners.add(l);
    }
  }

  /**
   * Removes a listener.
   * @param l the listener
   */
  public void removeSimulatorListener(SimulatorListener l) {
    listeners.remove(l);
  }

  public abstract void write(String data);
  public abstract void write(int data);
} // DeviceSimulator