package edu.ucf.cs.dml.simnets.measureme.devices;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.Vector;

public class DeviceSimulatorGUI extends JFrame {
  protected Vector listeners = new Vector();
  protected JTextArea statusArea = new JTextArea();
  protected JLabel statusLabel = new JLabel();
  protected JLabel fieldLabel = new JLabel();
  protected JTextField userValueField = new JTextField();
  protected JButton okButton = new JButton();
  protected JScrollPane scrollPane = new JScrollPane(statusArea);

  protected void jbInit() throws Exception {
    setDefaultCloseOperation(3);
    setResizable(false);
    getContentPane().setLayout(new FlowLayout());

    statusArea.setEditable(false);
    statusArea.setMinimumSize(new Dimension(300, 100));
    statusArea.setPreferredSize(new Dimension(300, 100));

    userValueField.setMinimumSize(new Dimension(100, 25));
    userValueField.setPreferredSize(new Dimension(100, 25));

    statusLabel.setText("Status: ");
    fieldLabel.setText("User Value: ");
    okButton.setText(" OK ");

    okButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        okButton_actionPerformed(e);
      }
    });

    getContentPane().add(fieldLabel);
    getContentPane().add(userValueField);
    getContentPane().add(okButton);
    getContentPane().add(statusLabel);
    getContentPane().add(scrollPane);

    pack();
  }

  public DeviceSimulatorGUI(String title) {
    super(title);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /**
   * Fires event <code>e</code>.
   * @param e the event
   */
  protected void fireSimulatorGUIEvent(SimulatorGUIEvent e) {
    for (int i = 0; i < listeners.size(); i++) {
      ((SimulatorGUIListener) listeners.elementAt(i)).guiEvent(e);
    }
  }

  /**
   * Adds a listener.
   * @param l the listener
   */
  public void addSimulatorGUIListener(SimulatorGUIListener l) {
    if (!listeners.contains(l)) {
      listeners.add(l);
    }
  }

  /**
   * Removes a listener.
   * @param l the listener
   */
  public void removeSimulatorGUIListener(SimulatorGUIListener l) {
    listeners.remove(l);
  }

  /**
   * Appends <code>message</code> to the status text area in the GUI.
   * @param message
   */
  public void appendStatus(String message) {
    statusArea.append(message+"\n");
  }

  /**
   * Fire GUI event.  Make sure userValue is a number first.
   * @param e
   */
  void okButton_actionPerformed(ActionEvent e) {
    float userValue;
    try {
      userValue = Float.parseFloat(userValueField.getText());
    }
    catch (NumberFormatException e1) {
      JOptionPane.showMessageDialog(this, "Value in field is not a number.");
      return;
    }
    fireSimulatorGUIEvent(new SimulatorGUIEvent(this, new Float(userValue)));
  }

} // DeviceSimulatorGUI