package edu.ucf.cs.dml.simnets.measureme.devices;

import edu.ucf.cs.dml.simnets.measureme.client.FootLengthValueMap;

public class FootLengthSimulator extends GrayCodeDeviceSimulator {

  public FootLengthSimulator() {
    super();
  }

  /**
   * Override parent's method to make sense for FootLength.
   * @return the default value
   */
  protected String getDefaultValue() {
    return compute(new Float(12.0F));
  }

  protected String compute(Object userValue) {
    /**
     * The calculation that occurs here undoes all calculations performed by
     * the device class (callback method), the device proxy (propertyChange and
     * applyOffset methods), and value map methods used in MeasureMeClient.
     */

    float lengthInInches = ((Float) userValue).floatValue();

    // LOWEST_MEASUREMENT is the smallest value the device can measure;
    //    for instance, foot length device cannot measure things smaller than 6 inches
    float adjustedLengthInInches = lengthInInches - FootLengthValueMap.LOWEST_MEASUREMENT;

    // each tick of the device corresponds to some fraction of an inch
    short ticks = (short) Math.round(adjustedLengthInInches / FootLengthValueMap.INCHES_PER_BIT);

    // see FootLengthValueMap.applyOffset() for why adjustTicks is calculated
    short adjustedTicks = (short) (255 - ticks);

    // create char array for easy 3 char String creation
    char[] commandAndDataArray = new char[3];
    commandAndDataArray[0] = 0;   // command byte (not actually used)
    System.out.println("Command byte: "+(short)commandAndDataArray[0]);
    commandAndDataArray[1] = (char) binary_to_gray((adjustedTicks >> 8));
    System.out.println("First data byte: "+(short)commandAndDataArray[1]);
    commandAndDataArray[2] = (char) binary_to_gray((adjustedTicks << 8 >> 8));
    System.out.println("Second data byte: "+(short)commandAndDataArray[2]);
    String commandAndDataString = new String(commandAndDataArray);
    System.out.println("Command and data as a string: "+commandAndDataString);
    return commandAndDataString;
  }

} // FootLengthSimulator