package edu.ucf.cs.dml.simnets.measureme.devices;


import java.util.Enumeration;
import com.objectspace.jgl.*;
import java.beans.*;
import java.io.*;
/**
 * CLASS GrayCodeDevice: Adds fuctionality for gray code based devices.
 * @author Dean Reed, Shaun Murphy
 */
public class GrayCodeDevice extends OSCDevice
{

  /**
   * CONSTRUCTOR : Should be overriden by a sublclass that specifies login codes
   */
  public GrayCodeDevice() {
    super();
    System.out.println("GrayCodeDevice Constructed");
    //deviceStartLoginCodes=new int[]{6,6,2};
  }



  /**
   * OVERRIDE: additional gray code conversion, handles intermediate converion
   *  at the device level.
   */
  public void callBack(String parm1) {
    String tempInput;
    System.out.println("parm1 length = "+parm1.length());
    System.out.println("buffer length = "+buffer.length());

    // append buffer string to parm1 so now we have everything yet to be processed
    String yetToProcess = buffer + parm1;
    buffer = "";

    System.out.println("yetToProcess =");
    // figure out what the remainder is; this number of bytes will go in buffer
    //    to be fired when we have enough (3) bytes
    int remainder = yetToProcess.length() % 3;

    // process all complete 3-byte commands
    for (int i = 0; i < yetToProcess.length()-remainder; i++) {
      inputShort = new Short ((short)yetToProcess.charAt(i));
      // only convert data bytes, not command bytes
      if (((i+1) % 3) != 1) {
        inputShort = new Short ((short) this.gray_to_binary(inputShort.shortValue()));
      }
      tempInput = Short.toString(inputShort.shortValue());
      System.out.println("byte"+i+" = "+inputShort);
      queue.push(tempInput);

      // fire a propertyChange every 3 bytes
      if (((i+1) % 3) == 0) {
        propertyChangeListeners.firePropertyChange(
              new PropertyChangeEvent(this,"devStr",new Boolean(true),
              new Boolean(false)));
      }
    }

    // store the rest in the buffer
    System.out.println("storing "+remainder+" bytes in buffer");
    buffer.concat(yetToProcess.substring(yetToProcess.length()-remainder));
  }

  /**
 * PURPOSE: Convert an integer in gray code to binary equivelent
 * @author Shaun Murphy
 */
  protected int gray_to_binary (int grayNumber)
  {
      int  n;
      int j;
      char highBit;
      int binNumber;
      String tempGrayNumber;
      String tempBinNumber;

     tempGrayNumber = Integer.toBinaryString(grayNumber);
     n = tempGrayNumber.length();
     j = tempGrayNumber.charAt(0);


     tempBinNumber = tempGrayNumber.charAt(0) + "";
     for (j = 1; j < n; j++)
     { tempBinNumber = tempBinNumber + (((int)tempBinNumber.charAt(j-1) - 48) ^ ((int)tempGrayNumber.charAt(j)) -48);
     }

     binNumber = Integer.valueOf(tempBinNumber, 2).intValue();

     return binNumber;

 }

  public String toString() {
    return super.toString() + " Gray Code Device ";
  }

  public boolean syncDevice() {
    return true;
  }

  public static void main(String[] args) {
    GrayCodeDevice d = new GrayCodeDevice();
//    System.out.println(d.gray_to_binary(14));    // expect decimal 11
//    System.out.println(d.gray_to_binary(2));     // expect decimal 3
    System.out.println(d.gray_to_binary(255-192));     // expect decimal 6
    System.out.println(d.gray_to_binary(255-191));     // expect decimal 6
//
  }

} // GrayCodeDevice