package edu.ucf.cs.dml.simnets.measureme.devices;
import java.beans.PropertyChangeEvent;
import java.io.*;
import edu.ucf.cs.dml.simnets.measureme.*;

/**
 * Title:        Measure Me client side classes/source
 * Description:  The source code for the measure me orlando science center project.
 *               Encapsulates some of the common behavior with osc graycode devices.
 * Copyright:    Copyright (c) 1999
 * Company:      DML
 * @author Dean Reed
 * @version
 */

public  class GrayCodeDeviceProxy extends DeviceProxy
{ Short value=null;  // the last value
  GrayCodeCalibrationData calibrationData=null;
  protected boolean newValueFromDevice=false;
  protected boolean started=false;

  public GrayCodeDeviceProxy()
  { super();
    this.calibrationData= this.getCalibrationCodes();

    this.value=new Short((short)0);
  }

  public GrayCodeDeviceProxy(DeviceInterface deviceInterface)
  { super(deviceInterface);
    this.value=new Short((short)0);
    this.calibrationData= this.getCalibrationCodes();

  }


  public void propertyChange(PropertyChangeEvent evt)
  { String currentByte=null;
    if(this.deviceInterface!=null) {
      for(int i=0; i<3; i++) {
        if(i==0) {
          try {
            currentByte= this.deviceInterface.readFromDevice();  //read the command byte
            System.out.println("Command byte: " + currentByte);

          }
          catch(Exception e) {

          }
        }
        else {
          if(i==1) {
            try{
              this.value=new Short(this.deviceInterface.readFromDevice()); //read in the high order bits
              System.out.println("2:" +this.value);
            }
            catch(Exception e2) {

            }
          }
          else {
            short temp=((short) (this.value.shortValue() <<8));
            try {
              value=new Short((short)(temp | Short.parseShort(this.deviceInterface.readFromDevice())));
              System.out.println("3:"+value);
            }
            catch(Exception e3) {

            }
          }
        }



      }
      this.newValueFromDevice=true;
      //this.notifyAll();

    }

  } // propertyChange

  /**
   * Block until device acknowledges.
   */
  public void waitForDeviceMeasure() throws DeviceFailureException,
                                                         InterruptedException {
    super.waitForDeviceMeasure();
    this.newValueFromDevice = false;
    this.deviceInterface.start();
    while (!newValueFromDevice && !stopped) {
      System.out.println("GrayCodeDeviceProxy.waitForDeviceMeasure() waiting");
      synchronized(this) {wait(MeasureMeGlobal.WAIT_TIME);}
    }
    if (stopped) {
      deviceInterface.stop();
    }



  /*
    try{
        if(!started)
         this.deviceInterface.start();
        started=true;

      }
     catch(Exception e2)
     {e2.printStackTrace();
     }

    try
    {

      while(!newValueFromDevice)
      { wait();

      }
    }
    catch(Exception e)
    {
      if (e instanceof InterruptedException)
        throw (InterruptedException) e;
      else
        e.printStackTrace();
    }
*/
  }

  /**
   * Take the first value we get when measure is called.
   */
  public Object measure() throws DeviceFailureException, InterruptedException
  {

    System.out.println("before: " + this.value + " after: " +this.applyOffset(this.value));
    //this.deviceInterface.stop();
    //this.started=false;
    deviceInterface.stop();

    // Edit M. Lowery 2002-05-17 (added stopped=true so waitForDeviceMeasure exits)
    this.stopped = true;
    return this.applyOffset(this.value);
  }


  public Object measureRaw() throws DeviceFailureException, InterruptedException {
    this.newValueFromDevice=false;
    if(!this.started) //need to start the device
    {
      this.deviceInterface.start();
      //then wait for a value
      while (!newValueFromDevice && !stopped) {
        synchronized(this) {
          wait(MeasureMeGlobal.WAIT_TIME);
        }
      }
    }
    System.out.println("before: " + this.value + " after: " +this.applyOffset(this.value));
    this.deviceInterface.stop();
    this.started=false;
    return this.value;
  }



  /**
   * PURPOSE: try to get calibration codes from the disk
   */
  protected GrayCodeCalibrationData getCalibrationCodes()
  {
    GrayCodeCalibrationData returnData=null;
    File f=new File("C:\\measureme\\greycodecalibration.ser");
    if(f.exists()) //then a calibration file exists
    { System.out.println("Calibration file found");
      try{ FileInputStream fis=new FileInputStream(f);
           ObjectInputStream ois=new ObjectInputStream(fis);
           returnData= (GrayCodeCalibrationData) ois.readObject();
           fis.close();
           return returnData;
      }
      catch(Exception e)
      {e.printStackTrace();

       return new GrayCodeCalibrationData((short)0);
      }
    }
    else
    { System.out.println("No calibration codes found");

    }
     return new GrayCodeCalibrationData((short)0);;

  }

  /**
   * PURPOSE: Apply modulo/ calibration to input values
   */
  protected  Short applyOffset(Short deviceValue)
  { if(this.calibrationData==null)
     return deviceValue;

    if(deviceValue==null)
     System.out.println("Error, device value is null");




    if( (deviceValue.shortValue()>= this.calibrationData.getZeroValue()) && (deviceValue.shortValue()<=255))

    {
     return new Short((short)(deviceValue.shortValue() -this.calibrationData.getZeroValue()));

    }

    return new Short((short)(255 + deviceValue.shortValue() - this.calibrationData.getZeroValue()));
  }
}