package edu.ucf.cs.dml.simnets.measureme.devices;

import java.util.Enumeration;
import com.objectspace.jgl.*;
import java.beans.*;
import java.io.*;
import edu.ucf.cs.dml.simnets.measureme.MeasureMeGlobal;

/**
 * CLASS HeightDevice: communication class for ULI height device.
 * @author Shaun Murphy 6/23/00
 */
public class HeightDevice extends Device
{
  private transient SerialParameters parameters;
  private DeviceConnection connection;
  private transient Queue queue;
  private int inputInt;
  private float inputFloat;
  public static final float speedOfSound = 343.0f;

  /**
  * Default constructor
  */
  public HeightDevice() {
    super();
    System.out.println("Height Constructed");
  }

  /**
  *  Purpose: creat connection and parameters objects and attempt to open,
  *  reset, and close device
  */
  public void initialize() throws DeviceFailureException
  {
    this.parameters = new SerialParameters();

    // determine whether to instantiate device or simulator

    String value = System.getProperty("runOption");
    int runOption = 0;
    if (value == null) {
      System.out.println("No runOption parameter found.  Defaulting to serial device.");
      runOption = MeasureMeGlobal.USE_PHYSICAL_DEVICE;
    }
    else {
      // try to convert from String to int
      try {
        runOption = Integer.parseInt(value);
        System.out.println("Successfully parsed runOption parameter: "+runOption);
      }
      catch (NumberFormatException e) {
        System.out.println("Unrecognized value for runOption parameter on command-line!");
        // if property not an int then default to serial connection
        runOption = MeasureMeGlobal.USE_PHYSICAL_DEVICE;
      }
    }

    if (runOption == MeasureMeGlobal.USE_DEVICE_SIMULATOR) {
      System.out.println("DEBUG - Using device simulator...");
      connection = new SimulatorConnection(this, new HeightSimulator());
    }
    else {
      System.out.println("DEBUG - Using serial device...");
      connection = new SerialConnection(this, parameters);
    }



    queue = new Queue();

    try{
    connection.openConnection();
    } catch (ConnectionException e2) {
    return;
    }

    connection.sendBreak();
    connection.sendData(" ");
    connection.sendData("d\r");
    connection.sendData("s1\r"); //note: make sure Motion sensor is in right port
    connection.sendData("t000187\r"); //note: most stable sample
    connection.closeConnection();
  }

  /**
  *  Purpose: open the serial port for device read/write
  */
  public void start() throws DeviceFailureException
  {System.out.println("device starting");
    try{
    connection.openConnection();
    } catch (ConnectionException e2) {
    return;
    }
    connection.sendData("m2\r");
    System.out.println("start done");
  }

  /**
  *  Purpose: terminate polling of device and close connection
  */
  public void stop() throws DeviceFailureException
  {
    System.out.println("device stopping");
    connection.sendData("\u0003");
    connection.closeConnection();
  }

  /**
  *  Purpose: send back recieved data buffer
  */
  public void callBack(String inputString)
  {
   System.out.println("Mat - callBack says: "+inputString);

   int inputStringLen = inputString.length();
   String inputStringValue = inputString.substring(0, (inputStringLen - 2) );
   //System.out.println(inputStringValue);
   try{
    inputInt = Integer.parseInt(inputStringValue);
    } catch (Exception e) {}
   System.out.println(inputInt);
   if (inputInt != 0){
    inputFloat = (float)(((((inputInt)/2.0))/1000000)*speedOfSound); // divide to get 1 way, convert to seconds, convert to meters.
   String f = new String(Float.toString(inputFloat));
   System.out.println(f);
   queue.push(f);
   synchronized(this){
   propertyChangeListeners.firePropertyChange(new PropertyChangeEvent(this,"devStr",new Boolean(true) , new Boolean(false)));
   }
   }
   }
   //}
  /**
  *  Purpose: to read from the FIFO queue on request
  */
  public String readFromDevice()
  {
   return queue.pop().toString();
  }

  /**
  *   Purpose: directly write to serial connection
  */
  public void writeToDevice(String deviceDataString) throws DeviceDataException
  {
    connection.sendData(deviceDataString);
  }

  /**
  *  Purpose: implement writeToDevice(Str) with protocol
  *  TODO: implement
  */
  public void writeToDevice(String deviceDataString, String protocolString) throws DeviceDataException, UnknownProtocolException
  {
  }

  /**
  *  Purpose: get available protocols
  *  TODO: implement
  */
  public Enumeration getProtocolCmds()
  {
   return null;
  }

  void writeObject(ObjectOutputStream oos) throws IOException
  { System.out.println("k");
    oos.defaultWriteObject();
  }

  void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException
  {
    ois.defaultReadObject();
  }
  public String toString()
  { return super.toString()+ " "+ this.parameters.getBaudRate()+"\n"+
     this.parameters.getBaudRateString();


  }

}
