package edu.ucf.cs.dml.simnets.measureme.devices;

public class HeightSimulator extends DeviceSimulator {
  protected long sampleInterval;
  public static final String START_SIGNAL = "m2\r";
  public static final String END_SIGNAL = "\u0003";
  protected boolean stopFlag;
  protected String currentSimValue;

  public HeightSimulator() {
    super();
    currentSimValue = getDefaultValue();
  }

  public void write(int data) {
//    do nothing for Height
  }

  public void write(String data) {
    // time parameter
    if (data.startsWith("t")) {
      int endIndex = data.indexOf("\r");
      sampleInterval = Long.parseLong(data.substring(1, endIndex));
      System.out.println("Sample interval: "+sampleInterval);
      simGUI.appendStatus("Received initialization parameters...");
      return;
    }
    // start a thread so this method call returns quickly
    if (data.equals(START_SIGNAL)) {
      simGUI.appendStatus("Received start signal...");
      stopFlag = false;
      new Thread(new Runnable() {
        public void run() {
          while (stopFlag != true) {
            fireSimulatorEvent(new SimulatorEvent(HeightSimulator.this, currentSimValue));
            simGUI.appendStatus("Sent "+currentSimValue+"...");
            try {
              Thread.currentThread().sleep(sampleInterval);
            }
            catch (InterruptedException e) { }
          }
        }
      }).start();
      return;
    }
    if (data.equals(END_SIGNAL)) {
      simGUI.appendStatus("Received stop signal...");
      stopFlag = true;
      return;
    }
  } // write

  public long getSampleInterval() {
    return sampleInterval;
  }

  public void guiEvent(SimulatorGUIEvent e) {
    // object at this point is a valid float of height in inches
    Float heightInInches = (Float) e.getData();
    currentSimValue = compute(heightInInches);
  }

  protected String getDefaultValue() {
    return compute(new Float(70.0F)); // 6 feet tall
  }

  protected String compute(Object userVal) {
    float heightInInches = ((Float) userVal).floatValue();
    float heightInMeters = heightInInches / HeightDeviceProxy.metersToInchesConversionFactor;
    // distanceToHead is the distance from sensor to visitor's head; take into account errorVal
    float distanceToHead = HeightDeviceProxy.distToTheFloor - heightInMeters + HeightDeviceProxy.errorVal;
    float oneWayTravelTime = distanceToHead / HeightDevice.speedOfSound;
    // double the one way time then multiply times 10^6 to get microseconds
    float roundTripTimeToHead = oneWayTravelTime * 2 * 1000000;
    // last two symbols get chopped off anyway (in HeightDevice)
    //   application expects microsecond count as an integer
    return String.valueOf((int) roundTripTimeToHead) + "--";
  }

} // HeightSimulator