package edu.ucf.cs.dml.simnets.measureme.devices;
import java.beans.*;
import edu.ucf.cs.dml.simnets.measureme.*;

/**
 * Title:        Devices for Measure Me
 * Description:  Device Wrapppers/Code for measure me.
 * Copyright:    Copyright (c) 1999
 * Company:      DML
 * @author Dean Reed
 * @version
 */

public class JumpDeviceProxy extends DeviceProxy {
  boolean reaching = false;
  boolean doneJumping = false;
  boolean runningJump = false;
  Short reach = new Short((short)0);
  Short jump = new Short((short)0);

  public JumpDeviceProxy() {
    super();
  }

  public JumpDeviceProxy(DeviceInterface deviceInterface) {
    super(deviceInterface);
  }

  public void propertyChange(PropertyChangeEvent evt) {
    Short temp;
    System.out.println("propertyChange: event");
    try {
      String cmdValue = this.deviceInterface.readFromDevice();
      switch (Integer.parseInt(cmdValue)) {
        case 2:
          // consume following 2 bytes (meaningless)
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();

          // reset value
          reach = new Short((short)0);
          doneJumping = false;
          runningJump = false;

          break;
        case 8:
          // reach data
          deviceInterface.readFromDevice();   // ignore first byte

          temp = new Short(deviceInterface.readFromDevice());
          if (temp.compareTo(reach) > 1) {
            reach = temp;
          }

          reaching = true;
          System.out.println("exhibitor started reaching");
          //this.notifyAll();
          break;
        case 18:
          // jump data
          deviceInterface.readFromDevice();
          jump = new Short(deviceInterface.readFromDevice());

          doneJumping = true;
          System.out.println("got jump data");
          //this.notifyAll();
          break;
        case 9:
          // running jump
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();
          runningJump = true;
          System.out.println("got running jump signal");
          //this.notifyAll();
          break;
        default: System.out.println("unknown command byte: "+ cmdValue);
          deviceInterface.readFromDevice();
          deviceInterface.readFromDevice();
          break;
      } // switch
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  } // propertyChange


  /**
   * Block until first reach data is received.
   */
  public void waitForDeviceMeasure() throws DeviceFailureException,
                                                         InterruptedException {
    super.waitForDeviceMeasure();
    reaching = false;

    this.deviceInterface.start();
    while (!reaching && !stopped) {
      System.out.println("JumpDeviceProxy.waitForDeviceMeasure() waiting");
      synchronized(this) {wait(MeasureMeGlobal.WAIT_TIME);}
    }
    if (stopped) {
      deviceInterface.stop();
    }
  } // waitForDeviceMeasure

  /**
   * Blocks until jump data is received.
   */
  public Object measure() throws DeviceFailureException,
                                              InterruptedException {
    while (!doneJumping && !runningJump && !stopped) {
      System.out.println("JumpDeviceProxy.measure() waiting");
      synchronized(this) {wait(MeasureMeGlobal.WAIT_TIME);}
    }
    deviceInterface.stop();

    if (runningJump) {
      return new Short((short)-1);
    }

    Short temp = new Short((short)(reach.shortValue() - jump.shortValue()));

    if (temp.shortValue() < 0) {
      return new Short((short)-1);
    }
    return temp;
  } // measure

  public void setChaseMode()
  { try
    {
      this.deviceInterface.writeToDevice("Chase");
    }
    catch(Exception e)
    { e.printStackTrace();
    }

  }

} // JumpDeviceProxy