package edu.ucf.cs.dml.simnets.measureme.devices;

public abstract class LoadCellSimulator extends OSCDeviceSimulator {
  protected long timeForActivity;
  protected int halfPounds;
  protected int DATA_INDICATOR;    // subclass must define
  protected int STOP_INDICATOR;     // subclass must define
  protected final long sampleInterval = 500;

  public void guiEvent(SimulatorGUIEvent e) {
    // object at this point is a valid float of time in seconds; convert to ms
    Number[] vals = (Number[]) e.getData();
    float timeInSeconds = vals[0].floatValue();
    int pounds = vals[1].intValue();
    timeForActivity = (long) (timeInSeconds * 1000);
    halfPounds = pounds * 2;
  }

  public void write(int data) {
    if ((++loginCodeByteCounter % 3) == 0) {
      // start thread here to allow app to continue processing device signals
      new Thread(new Runnable() {
        public void run() {
          simGUI.appendStatus("Sim: Got login code...");
          loginCodeByteCounter = 0;
          char[] commandAndDataArray = new char[3];
          String commandAndDataString;
          commandAndDataString = new String(OSCDeviceSimulator.ACK_SIGNAL);
          simGUI.appendStatus("Sim: Sending ack signal...");
          fireSimulatorEvent(new SimulatorEvent(this, commandAndDataString));

          // pause for one sec just for reading screen
          try {
            Thread.currentThread().sleep(1000);
          }
          catch (InterruptedException e) { }

          long timeToStop = System.currentTimeMillis() + timeForActivity;
          commandAndDataArray[0] = (char) DATA_INDICATOR;
          commandAndDataArray[1] = (char) (halfPounds >> 8);
          commandAndDataArray[2] = (char) (halfPounds << 8 >> 8);
          commandAndDataString = new String(commandAndDataArray);
          while (System.currentTimeMillis() < timeToStop) {
            simGUI.appendStatus("Sim: Sending value "+halfPounds);
            fireSimulatorEvent(new SimulatorEvent(this, commandAndDataString));
            try {
              Thread.currentThread().sleep(sampleInterval);
            }
            catch (InterruptedException e) { }
          }
          commandAndDataArray[0] = (char) STOP_INDICATOR;
          commandAndDataString = new String(commandAndDataArray);
          simGUI.appendStatus("Sim: Sending stop signal...");
          fireSimulatorEvent(new SimulatorEvent(this, commandAndDataString));
        } // run
      }).start(); // Thread
    } // if
  }
} // LoadCellSimulator