
/**
 * Title:        Devices for Measure Me<p>
 * Description:  Device Wrapppers/Code for measure me.<p>
 * Copyright:    Copyright (c) 1999<p>
 * Company:      DML<p>
 * @author Dean Reed
 * @version
 */
package edu.ucf.cs.dml.simnets.measureme.devices;
import com.objectspace.jgl.Queue;
import java.util.Enumeration;
import java.beans.*;
import java.io.*;
import java.util.StringTokenizer;
import edu.ucf.cs.dml.simnets.measureme.MeasureMeGlobal;

/**
 * CLASS OSCDevice: Encapsulates many of the services/operations
 *  needed for the devices built at osc
 */
public abstract class OSCDevice extends Device
{ private transient SerialParameters parameters; //parameters specifically for the com port
  private DeviceConnection connection;
  protected transient Queue queue;                // incoming data storage
  protected int inputInt;                         //temp value
  protected float inputFloat;                     //temp value
  protected Short inputShort;                     //temp value
  //protected boolean killStart = false;            //
  protected transient String buffer = "";


  protected int[] deviceStartLoginCodes=null;  //codes to tell the device that a client has started


  public OSCDevice() {
    super();
    System.out.println("OSC Device Constructed");
  }

  /**
   * Edit: M. Lowery 2001-03-02
   * If callBack gets a parm1 that does not have length that is a multiple of 3,
   * fire propertyChanges for the complete chunks and store the remainder for
   * the next callBack.
   */
  public void callBack(String parm1)
  {
    String tempInput;
    System.out.println("parm1 length = "+parm1.length());
    System.out.println("buffer length = "+buffer.length());

    // append buffer string to parm1 so now we have everything yet to be processed
    String yetToProcess = buffer + parm1;
    buffer = "";

    System.out.println("yetToProcess =");
    // figure out what the remainder is; this number of bytes will go in buffer
    //    to be fired when we have enough (3) bytes
    int remainder = yetToProcess.length() % 3;

    // process all complete 3-byte commands
    for (int i = 0; i < yetToProcess.length()-remainder; i++) {
      inputShort = new Short ((short)yetToProcess.charAt(i));
      tempInput = Short.toString(inputShort.shortValue());
      System.out.println("byte"+i+" = "+inputShort);
      queue.push(tempInput);

      // fire a propertyChange every 3 bytes
      if (((i+1) % 3) == 0) {
        propertyChangeListeners.firePropertyChange(
              new PropertyChangeEvent(this,"devStr",new Boolean(true),
              new Boolean(false)));
      }
    }

    // store the rest in the buffer
    System.out.println("storing "+remainder+" bytes in buffer");
    buffer.concat(yetToProcess.substring(yetToProcess.length()-remainder));


/*
     for(int i=0;i<parm1.length();i++)
      { //inputChar = parm1.charAt(i);

        inputShort = new Short ((short)parm1.charAt(i));


        System.out.println(inputShort);
        tempInput = Short.toString(inputShort.shortValue());

        queue.push(tempInput);

      }




   propertyChangeListeners.firePropertyChange(new PropertyChangeEvent(this,"devStr",new Boolean(true) , new Boolean(false)));
*/
   //System.out.println("fired "+parm1);
    //TODO: implement this edu.ucf.cs.dml.simnets.measureme.devices.Device abstract method
  }

  // ********************************************************
  // EDIT: M. Lowery (2001-01-26)
  // CHANGE: instead of catching all exceptions, we need to
  //         re-throw InterruptedExceptions
  // ********************************************************

  public String readFromDevice() //throws java.lang.InterruptedException
  { if(queue.isEmpty()) {
      System.out.println("Mat says this can't happen -- read queue empty");
      return "0"; // this is because a wait scares CEH
    }
/*  while (queue.isEmpty()) {
      { try{
      System.out.println("Mat - OSCDevice.readFromDevice(): Waiting for 125 ms (empty queue)...");
      wait(125);
          System.out.println("Trying to pop an empty queue");
         }
     catch(Exception e)
     {
       if (e instanceof InterruptedException)
         throw (InterruptedException) e;
       else
         e.printStackTrace();
     }
    }
*/

   return queue.pop().toString();
  }

  public Enumeration getProtocolCmds()
  { return null;

  }

  public void writeToDevice(String parm1, String parm2) throws DeviceDataException, UnknownProtocolException
  {
    //TODO: implement this edu.ucf.cs.dml.simnets.measureme.devices.Device abstract method
  }

  public void start() throws DeviceFailureException
  { System.out.println("device starting");
    if(this.deviceStartLoginCodes==null)
     throw new DeviceFailureException("Start codes are null at start");
    if(connection==null)
     throw new DeviceFailureException("Connection null in start");
    try
    { if(!connection.isOpen())
       connection.openConnection();

        System.out.println("opened connection");
    } catch (ConnectionException e2)
    {  e2.printStackTrace();
    }
    if(this.deviceStartLoginCodes==null)
     System.out.println("NULL DEVICE CODES");
    if(this.connection==null)
     System.out.println("NULL CONNECTION");

    // Edit M. Lowery 2001-03-02 Make sure we clear out queue

    queue.clear();
    buffer = "";
    /////////////////////////////////////////////////////////////////////
    String startCode = "";
    System.out.println("sending login codes to device...");
    for( int i=0; i<this.deviceStartLoginCodes.length;i++)  {//send the login signal
      startCode = startCode + " " + String.valueOf(deviceStartLoginCodes[i]);
      this.connection.sendData(this.deviceStartLoginCodes[i]);
    }
    //this.killStart=false;
   System.out.println("OSC Device started; sent: " + startCode);
  }

 /**
  *   Purpose: directly write to serial connection
  */
  public void writeToDevice(String parm1) throws DeviceDataException
  { if(this.connection==null)
     throw new DeviceDataException("Null connection in write to device","Use start before trying to write");
    if(!connection.isOpen())
      throw new DeviceDataException("Connection closed in write to device","Use start before trying to write");
    connection.sendData(parm1);
   }

  public void stop() throws DeviceFailureException {
    System.out.println("device stopping");
    if(connection != null) {
      connection.closeConnection();
    }
    //killStart=true;  //added new
  }

  public void initialize() throws DeviceFailureException
  { System.out.println("initializing osc device");
    this.parameters = new SerialParameters("COM1",
	     9600,
	     2,
	     2,
	     8,
	     1,
	     0);
//    System.out.println("device constructed with new serial params");

    System.out.println("**********************************************");
    System.out.println("Note: To use a device simulator, add");
    System.out.println("-DrunOption=2");
    System.out.println("to the command line.");
    System.out.println("To use real device, remove that parameter.");
    System.out.println("**********************************************");

    // determine whether to instantiate device or simulator
    String value = System.getProperty("runOption");
    int runOption = 0;
    if (value == null) {
      System.out.println("No runOption parameter found.  Defaulting to serial device.");
      runOption = MeasureMeGlobal.USE_PHYSICAL_DEVICE;
    }
    else {
      // try to convert from String to int
      try {
        runOption = Integer.parseInt(value);
        System.out.println("Successfully parsed runOption parameter: "+runOption);
      }
      catch (NumberFormatException e) {
        System.out.println("Unrecognized value for runOption parameter on command-line!");
        // if property not an int then default to serial connection
        runOption = MeasureMeGlobal.USE_PHYSICAL_DEVICE;
      }
    }

    if (runOption == MeasureMeGlobal.USE_DEVICE_SIMULATOR) {
      System.out.println("DEBUG - Using device simulator...");
      // now determine what simulator to instantiate
      StringTokenizer st = new StringTokenizer(MeasureMeGlobal.stationName);
      StringBuffer noSpaces = new StringBuffer();
      while (st.hasMoreTokens()) {
        noSpaces.append(st.nextElement());
      }
      Class simClass = null;
      try {
        simClass = Class.forName("edu.ucf.cs.dml.simnets.measureme.devices."+noSpaces+"Simulator");
        connection = new SimulatorConnection(this, (DeviceSimulator) simClass.newInstance());
      }
      catch (Exception e) {
        e.printStackTrace();
      }
    }
    else {
      System.out.println("DEBUG - Using serial device...");
      connection = new SerialConnection(this, parameters);
    }


//    connection = new SerialConnection(this, this.parameters);

    if(this.queue==null)
      queue = new Queue();
    else  //dump any que contenets
    { while(!this.queue.isEmpty())
        this.queue.pop();
    }
    try{
      connection.openConnection();
    }
    catch (ConnectionException e2) {
      e2.printStackTrace();
      return;
    }
    if(!this.syncDevice())
     throw new DeviceFailureException("Device not syncronized");
  }

  public String toString()
  { String retString="OSCDevice: "+super.toString()+ " ";
    if(this.parameters!=null)
    { retString+=this.parameters.getBaudRate()+"\n"+
      this.parameters.getBaudRateString();
    }
    else
     retString+="\n null parameters";
    return retString;


  }

  /**
   * PURPOSE: Synchronize with the current device
   *  Return value is true iff the device is in a syncronous state
   */
  protected abstract boolean syncDevice();


}