package edu.ucf.cs.dml.simnets.measureme.devices;
import javax.comm.*;
import java.io.*;
import java.awt.event.*;
import java.util.TooManyListenersException;
/**
 * CLASS Device: class that handles the details of a serial connection.
 * Holds the state of the connection.
 * @author Shaun Murphy 6/23/00
 */

 public class SerialConnection extends DeviceConnection implements SerialPortEventListener,
                                				   CommPortOwnershipListener {
    private ConnectionListener parent;
    private SerialParameters parameters;
    private String outputString;
    private int outputInt;
    private String inputString;
    private transient OutputStream os;
    private transient InputStream is;
    private transient CommPortIdentifier portId;
    private transient SerialPort sPort;
    private boolean open;
    /**
    Creates a SerialConnection object and initilizes variables passed in
    as params.

    @param parent A Device object.
    @param parameters A SerialParameters object.
    */
  public SerialConnection(ConnectionListener parent, SerialParameters parameters)
	{
	this.parent = parent;
	this.parameters = parameters;
  open = false;
  }
   /**
   *  Purpose:  Attempts to open a serial connection and streams using the parameters
   *  in the SerialParameters object. If it is unsuccesfull at any step it
   *  returns the port to a closed state, throws a
   *  <code>SerialConnectionException</code>, and returns.
   *  Gives a timeout of 30 seconds on the portOpen to allow other applications
   *  to reliquish the port if have it open and no longer need it.
   */
   public void openConnection() throws SerialConnectionException {

        // call closeConnection just in case it didn't get called previously
        this.closeConnection();

        // Edit 2002-Aug-15 M. Lowery
        // Added following code so that the following error would not occur
        // "Caught java.lang.NullPointerException: name can't be null while
        // loading com.sun.comm.Win32Driver"
        String driverName = "com.sun.comm.Win32Driver";
        try {
          CommDriver driver = (CommDriver) Class.forName(driverName).newInstance();
          driver.initialize();
        }
        catch (Throwable th) {

        }
        // end edit 2002-Aug-15

	// Obtain a CommPortIdentifier object for the port you want to open.
	try {
          System.out.println("Requesting port id object for "+parameters.getPortName());
	    portId =
		 CommPortIdentifier.getPortIdentifier(parameters.getPortName());
	} catch (NoSuchPortException e) {
      System.out.println("Someone Held port for > 30s!");
        e.printStackTrace();
	    throw new SerialConnectionException(e.getMessage());
	}

	// Open the port represented by the CommPortIdentifier object. Give
	// the open call a relatively long timeout of 30 seconds to allow
	// a different application to reliquish the port if the user
	// wants to.
	try {
	    sPort = (SerialPort)portId.open("Device", 30000);
	} catch (PortInUseException e) {
	    throw new SerialConnectionException(e.getMessage());

	}

	// Set the parameters of the connection. If they won't set, close the
	// port before throwing an exception.
	try {
	    setConnectionParameters();
	} catch (SerialConnectionException e) {
	    sPort.close();
	    throw e;
	}

	// Open the input and output streams for the connection. If they won't
	// open, close the port before throwing an exception.
	try {
	    os = sPort.getOutputStream();
	    is = sPort.getInputStream();
	} catch (IOException e) {
	    sPort.close();
	    throw new SerialConnectionException("Error opening i/o streams");
	}

	// Add this object as an event listener for the serial port.
	try {
	    sPort.addEventListener(this);
	} catch (TooManyListenersException e) {
	    sPort.close();
	    throw new SerialConnectionException("too many listeners added");
	}

	// Set notifyOnDataAvailable to true to allow event driven input.
	sPort.notifyOnDataAvailable(true);

	// Set notifyOnBreakInterrup to allow event driven break handling.
	sPort.notifyOnBreakInterrupt(true);

	// Set receive timeout to allow breaking out of polling loop during
	// input handling.
	try {
	    sPort.enableReceiveTimeout(30);
	} catch (UnsupportedCommOperationException e) {
	}

	// Add ownership listener to allow ownership event handling.
	portId.addPortOwnershipListener(this);

	open = true;
    }

    /**
    *  Purpose: Sets the connection parameters to the setting in the parameters object.
    *  If set fails return the parameters object to origional settings and
    *  throw exception.
    */
    public void setConnectionParameters() throws SerialConnectionException {

	// Save state of parameters before trying a set.
	int oldBaudRate = sPort.getBaudRate();
	int oldDatabits = sPort.getDataBits();
	int oldStopbits = sPort.getStopBits();
	int oldParity   = sPort.getParity();
	int oldFlowControl = sPort.getFlowControlMode();

	// Set connection parameters, if set fails return parameters object
	// to original state.
	try {
	    sPort.setSerialPortParams(parameters.getBaudRate(),
				      parameters.getDatabits(),
				      parameters.getStopbits(),
				      parameters.getParity());
	} catch (UnsupportedCommOperationException e) {
	    parameters.setBaudRate(oldBaudRate);
	    parameters.setDatabits(oldDatabits);
	    parameters.setStopbits(oldStopbits);
	    parameters.setParity(oldParity);
	    throw new SerialConnectionException("Unsupported parameter");
	}

	// Set flow control.
	try {
	    sPort.setFlowControlMode(parameters.getFlowControlIn()
			           | parameters.getFlowControlOut());
	} catch (UnsupportedCommOperationException e) {
	    throw new SerialConnectionException("Unsupported flow control");
	}
    }

    /**
    *  Purpose: Close the port and clean up associated elements.
    */
    public void closeConnection() {
	// If port is alread closed just return.
	if (!open) {
	    return;
	}

	// Check to make sure sPort has reference to avoid a NPE.
	if (sPort != null) {
	    try {
		// close the i/o streams.
	    	os.close();
	    	is.close();
	    } catch (IOException e) {
		System.err.println(e);
	    }

	    // Close the port.
	    sPort.close();

	    // Remove the ownership listener.
	    portId.removePortOwnershipListener(this);
	}
  open = false;
    }

   /**
   *  Purpose: Send String to sPort outputStream
   */

   public void sendData(String outputString){
   try{
   Thread.sleep(200);
   }catch (InterruptedException e4){}

   try{
   os.write(outputString.getBytes());
   }
   catch (IOException e) {
    e.printStackTrace();
   }
   };

   /**
   *  Purpose: Send Int to sPort outputStream
   */

   public void sendData(int outputInt){
   //try{
   //Thread.sleep(200);
   //}catch (InterruptedException e4){}
   if(os==null)
    System.out.println("NULL OS");
   try{
   os.write(outputInt);
   }
   catch (IOException e) {
    e.printStackTrace();
   }
   };


    /**
    *  Purpose: Send a one second break signal.
    */

    public void sendBreak() {
	  sPort.sendBreak(1000);
    }

    /**
    *  Purpose: Reports the open status of the port.
    *  @return true if port is open, false if port is closed.
    */
    public boolean isOpen() {
	return open;
    }

    /**
    *  Purpose: Handles SerialPortEvents. The two types of SerialPortEvents that this
    *  program is registered to listen for are DATA_AVAILABLE and BI. During
    *  DATA_AVAILABLE the port buffer is read until it is drained, when no more
    *  data is availble and 30ms has passed the method returns. When a BI
    *  event occurs the words BREAK RECEIVED are written to the messageAreaIn.
    */
    public synchronized void serialEvent(SerialPortEvent e) {
 	// Create a StringBuffer and int to receive input data.
	StringBuffer inputBuffer = new StringBuffer();
	int newData = 0;

	// Determine type of event.
	switch (e.getEventType()) {

	    // Read data until -1 is returned. If \r is received substitute
	    // \n for correct newline handling.
	    case SerialPortEvent.DATA_AVAILABLE:
		    while (newData != -1) {
		    	try {
		    	    newData = is.read();
			    if (newData == -1) {
				break;
			    }
			    //if ('\r' == (char)newData) {
          //} else if ('\n' == (char)newData) {
          //} else {
           //System.out.println((int)newData);

			    	inputBuffer.append((char)newData);
			    //}
		    	} catch (IOException ex) {
		    	    System.err.println(ex);
		    	    return;
		      	}
   		    }

    // Return collected data to be put into the FIFO queue
    try{
    //System.out.println("inputBuffer");
    parent.callBack(new String(inputBuffer));
    } catch (Exception e3) {

    }
    break;

	    // If break event append BREAK RECEIVED message.
	  case SerialPortEvent.BI:
	}
    // TODO: plugin new SerialPortEvents for future use.
    }

    /**
    Handles ownership events. If a PORT_OWNERSHIP_REQUESTED event is
    received a dialog box is created asking the user if they are
    willing to give up the port. No action is taken on other types
    of ownership events.
    */
    public void ownershipChange(int type) {
      // portId.open executed above; close port so open can proceed
      // openConnection executed but didn't get followed by closeConnection
      // this event signals contention, so simply give up port since
      // we're both the owner and current requestor
      if (type == CommPortOwnershipListener.PORT_OWNERSHIP_REQUESTED) {
        System.out.println("*******************PORT_OWNERSHIP_REQUESTED*****************");
        System.out.println("closing connection");
        this.closeConnection();
      }
    }

 void writeObject(ObjectOutputStream oos) throws IOException
 { open=false;
   oos.defaultWriteObject();

 }
 }

