package edu.ucf.cs.dml.simnets.measureme.devices;

/**
 * This class is analogous to <code>SerialConnection</code> for real devices.
 */
public class SimulatorConnection extends DeviceConnection implements SimulatorListener {
  /**
   * The object that implements <code>callback</code>.
   */
  protected ConnectionListener parent;

  /**
   * The object that understands and speaks that language of the device.  It responds
   * to login codes and fires byte sequences in the same way the real device does.
   */
  protected DeviceSimulator simulator;

  /**
   * Connection status.
   */
  protected boolean open;

  /**
   * Constructs a new simulator connection.
   * @param parent object that gets notified when data is available from
   * connection
   */
  public SimulatorConnection(ConnectionListener parent, DeviceSimulator sim) {
    this.parent = parent;
    this.simulator = sim;
    open = false;
  }

  /**
   * Starts listening to <code>DeviceSimulator</code>.
   * @throws SimulatorConnectionException
   */
  public void openConnection() throws SimulatorConnectionException {
    simulator.addSimulatorListener(this);
    open = true;
  }

  /**
   * Stops listening to <code>DeviceSimulator</code>.
   */
  public void closeConnection() {
    simulator.removeSimulatorListener(this);
    open = false;
  }

  /**
   * This method has no meaning for this class.
   * @throws SimulatorConnectionException
   */
  public void setConnectionParameters() throws SimulatorConnectionException {
//    do nothing
  }

  /**
   * This is used in devices like Height.
   * @param outputString data to send
   */
  public void sendData(String outputString) {
    simulator.write(outputString);
  }

  /**
   * This is used for device login codes.
   * @param outputInt data to send
   */
  public void sendData(int outputInt) {
    simulator.write(outputInt);
  }

  /**
   * Sends break signal.
   */
  public void sendBreak() {
    simulator.write(DeviceSimulator.BREAK_STRING);
  }

  /**
   * Returns the status of the connection
   * @return status
   */
  public boolean isOpen() {
    return open;
  }

  /**
   * Method defined in <code>SimulatorListener</code>.
   * @param e the simulator event
   */
  public synchronized void simulatorEvent(SimulatorEvent e) {
    parent.callBack((String) e.getData());
  }
} // SimulatorConnection