
/**
 * Title:        MeasureMe Server Side<p>
 * Description:  Server side code. <p>
 * Copyright:    Copyright (c) 2000<p>
 * Company:      DML<p>
 * @author Dean Reed
 * @version
 */
package edu.ucf.cs.dml.simnets.measureme.server;

import java.io.*;
import java.util.Date;
import java.util.Hashtable;
import java.text.SimpleDateFormat;

public class Exhibitor implements Serializable {
  protected String scanCode;      //the bar scanner of the exhibitor
  protected short age=-1;         //the age of the exhibitor
  protected java.awt.Color favoriteColor=null;
  protected short countryCode=-1;
  protected String state="NO";
  protected short countyCode=-1;
  protected Date dateVisited=null;
  protected short guideCharacterID=-1;
  protected short weight=-1;
  protected char gender='N';  //n is no gender provided
  protected short schoolCode=-1;
  protected int tycoonscore=-901;
  protected int heroscore=-1;

  //the scores object for each exhibit should go here
  protected Hashtable exhibitScores=new Hashtable();  //the scores key exhibitid, value-> ExhibitScoreInfo


  public Exhibitor(String scanCode)
  { this.scanCode=scanCode;
    this.dateVisited=new Date();

  }

  void readObject(ObjectInputStream ois) throws ClassNotFoundException, IOException
  {
    ois.defaultReadObject();
  }

  void writeObject(ObjectOutputStream oos) throws IOException
  {
    oos.defaultWriteObject();
  }

  public void setScanCode(String newScanCode)
  {
    scanCode = newScanCode;
  }

  public String getScanCode()
  {
    return scanCode;
  }

  public void setAge(short newAge)
  {
    age = newAge;
  }




  public short getAge()
  {
    return age;
  }

  public char getGender()
  { return this.gender;
  }

  public void setGender(char gender)
  { if((gender=='M') || (gender=='F') || (gender=='m') || (gender=='f'))
    { if( (gender=='m') || (gender=='f'))
       if(gender=='m')
        gender='M';
       else
        gender='F';

      this.gender=gender;
    }
    else
     System.out.println("Error, unrecognized gender in Exhibitor: setGender()");

  }

  public void setFavoriteColor(java.awt.Color newFavoriteColor)
  {
    favoriteColor = newFavoriteColor;
  }

  public java.awt.Color getFavoriteColor()
  {
    return favoriteColor;
  }

  public void setCountryCode(short newCountry)
  {
    countryCode = newCountry;
  }

  public short getCountryCode()
  {
    return countryCode;
  }

  public void setState(String newState)
  {
    state = newState;
  }

  public String getState()
  {
    return state;
  }

  public void setCountyCode(short newCountyCode)
  {
    countyCode = newCountyCode;
  }

  public short getCountyCode()
  {
    return countyCode;
  }

  public void setGuideCharacterID(short newGuideCharacterID)
  {
    guideCharacterID = newGuideCharacterID;
  }

  public short getGuideCharacterID()
  {
    return guideCharacterID;
  }

  public void setWeight(short weight)
  { this.weight=weight;
  }

  public short getWeight()
  { return this.weight;
  }

  public void setSchoolCode(short schoolCode)
  { this.schoolCode=schoolCode;
  }

  public short getSchoolCode()
  { return this.schoolCode;
  }

  public void setTycScore(int tycScore)
  { this.tycoonscore=tycScore;
  }
  public int getTycScore()
  { return this.tycoonscore;
  }
  public void setHeroScore(int heroScore)
  { this.heroscore=heroScore;
  }
  public int getHeroScore()
  { return this.heroscore;
  }



  /**
   * PURPOSE: return the score data for the requested station
   * @returns ExhibitScoreInfo relating to this exhibors score based on
   *   the exhibitID
   * @returns null if no exhibit data for the exhibit exists
   */
  public ExhibitScoreInfo getScoreInfo(short exhibitID)
  { return (ExhibitScoreInfo)this.exhibitScores.get(new Short(exhibitID));
  }

  /**
   * PURPOSE: add another exhibit data to this exhibitor
   */
  public void addExhibitScoreInfo(ExhibitScoreInfo info)
  {  if(this.getScoreInfo(info.getExhibitID())==null)
       this.exhibitScores.put(new Short(info.getExhibitID()),info);
     else
      System.out.println("Error: exhibit data already existed in the exibitor");

  }

  /**
   * PURPOSE: returns an sql query update string with this exhibitors data
   */
  public String getQueryUpdateString()
  { //generate an update string
    String updateString="INSERT INTO exhibitor VALUES(";  //we have multiple vals
    updateString+=this.scanCode+",";  //the scancode
    //need date in the form 9/11/00
    updateString+="'";
    java.text.SimpleDateFormat sdf=new SimpleDateFormat("MM/dd/yy");
    updateString+=sdf.format(this.dateVisited)+"',";
    updateString+=Short.toString(this.age)+",";
    updateString+=Short.toString(this.weight)+",";
    updateString+="'"+ this.gender+"',";
    updateString+=Short.toString(this.guideCharacterID)+",";
    //need to convert country code into
    updateString+=Short.toString(this.countryCode)+",";
    updateString+="'"+this.state+"',";
    updateString+=Short.toString(this.countyCode)+",";
    updateString+=Short.toString(this.schoolCode)+",";
    //now for the hard part.. need to build the substring of values
    updateString+="'"+this.getValuesString()+"',";
    //now compute the total number of stations visited
    updateString+=Integer.toString(this.exhibitScores.size())+",";
    updateString+=Integer.toString(this.tycoonscore)+ ",";
    updateString+=Integer.toString(this.heroscore);
    updateString+=");";  //whew

    return updateString;
  }

  protected String getValuesString()
  { String returnString="{";
    ExhibitScoreInfo info=null;
    for(int i=1;i<=25;i++)
    { info=(ExhibitScoreInfo)this.exhibitScores.get(new Short((short)i));
      if(info==null) //no visit to that station
       info=new ExhibitScoreInfo((short)i);
      returnString+=Short.toString(info.getScore());
      returnString+=",";
      returnString+=Short.toString(info.getNumberOfTimesVisited());
      returnString+=",";
      returnString+=Short.toString(info.getFirstVisitTime());
      returnString+=",";
      returnString+=Short.toString(info.getLastVisitTime());
      if(i!=25)
       returnString+=",";

    }
    returnString+="}";
    return returnString;
  }

  public String toString() {
    return "Exhibitor "+scanCode;
  }


}