package edu.ucf.cs.dml.simnets.measureme.server;

import edu.ucf.cs.dml.simnets.measureme.MeasureMeGlobal;
import net.jini.core.lookup.*;
import net.jini.core.discovery.*;
import net.jini.space.JavaSpace;
import java.io.*;
import java.net.*;
import java.rmi.*;

/**
 * Title:        MeasureMe Server Side
 * Description:  Server side code.
 * Copyright:    Copyright (c) 1999
 * Company:      DML
 * @author Mathew Lowery
 * @version
 */

/**
 * This class provides one static method that returns a handle to a JavaSpace
 * object via unicast lookup on a hard-coded server.  This class replaces
 * SpaceAccessor since classes and methods used in it have been deprecated
 * beginning in Jini 1.1.
 */
public class MMSpaceAccessor {

  public static JavaSpace getSpace() {
    LookupLocator lookup = null;
    ServiceRegistrar registrar = null;

    // set security manager
    if (System.getSecurityManager() == null) {
      System.out.println("setting new security manager");
      System.setSecurityManager(new RMISecurityManager());
    }

    // Edit M. Lowery 2002-Sep-13:  current config is unicast (see below)

    DiscoveryUtil d = new DiscoveryUtil();
    try {
//      registrar = d.discoverOne();              // multicast
      registrar = d.locate(MeasureMeGlobal.ipAddress);    // unicast
    }
    catch (JiniUtilException e) {
      e.printStackTrace();
      return null;
    }

    /*
    try {
      lookup = new LookupLocator("jini://pc176.cs.ucf.edu");
    }
    catch (java.net.MalformedURLException e) {
      System.out.println("Lookup failed: " + e.toString());
      return null;
    }


    try {
      System.out.println("finding lookup service");
      registrar = lookup.getRegistrar();
    }
    catch (java.io.IOException e) {
      System.out.println("Registrar search failed: " + e.toString());
      return null;
    }
    catch (java.lang.ClassNotFoundException e) {
      System.out.println("Registrar search failed: " + e.toString());
      return null;
    }
    */

    // look for JavaSpace service
    Class[] javaSpaceClasses = new Class[1];
    javaSpaceClasses[0] = JavaSpace.class;
    ServiceTemplate template = new ServiceTemplate(null, javaSpaceClasses, null);
    JavaSpace javaSpace = null;
    try {
      System.out.println("looking up JavaSpace");
      javaSpace = (JavaSpace) registrar.lookup(template);
    }
    catch(java.rmi.RemoteException e) {
      System.out.println("Error finding JavaSpace: " + e.toString());
      return null;
    }

    return javaSpace;
  } // getSpace


  public static void main(String[] args) {
    MMSpaceAccessor sa = new MMSpaceAccessor();
    JavaSpace js = sa.getSpace();
    System.out.println("got JavaSpace: "+js.toString());
  }
}