package edu.ucf.cs.dml.simnets.measureme.server;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: </p>
 * @author Mathew B. Lowery
 * @version 1.0
 */

import net.jini.core.lookup.ServiceID;
import net.jini.core.lookup.ServiceRegistration;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lease.Lease;
import java.util.Vector;
import java.rmi.RemoteException;
import net.jini.lease.LeaseRenewalManager;
import net.jini.lease.LeaseListener;
import net.jini.lease.LeaseRenewalEvent;
import net.jini.core.lease.UnknownLeaseException;
import java.rmi.RMISecurityManager;

/**
 * A convenience class for registering service objects.  This object adheres
 * to the convention that the same object should have the same ServiceID
 * in all lookup services in which it registers.  This class also maintains
 * all leases by renewing them every 10 minutes.
 * @see JiniUtilException
 * @see DiscoveryUtil
 * @see ServiceSearchUtil
 */
public class ServiceRegistrationUtil {
  private static final boolean DEBUG = true;
  private ServiceID serviceID = null;
  private final long LEASE_DURATION = 60*1000*10;  // 10 minutes
  private LeaseRenewalManager leaseMan = new LeaseRenewalManager();
  private Vector serviceRegistrations = new Vector();
  private Object obj = null;

  /**
   * Creates a utility object that can register and maintain leases for object
   * <code>obj</code>.
   * @param obj object to register
   */
  public ServiceRegistrationUtil(Object obj) {
    this.obj = obj;
  } // constructor

  /**
   * Registers object in lookup service running on <code>host</code>.  This
   * method performs the lookup discovery and object registration.
   * @param host machine running lookup service
   * @throws JiniUtilException
   */
  public void register(String host) throws JiniUtilException {
    DiscoveryUtil d = new DiscoveryUtil();
    ServiceRegistrar registrar = d.locate(host);
    if (registrar == null) {
      throw new JiniUtilException("lookup service on "+host+" not available");
    }
    // serviceID may or may not be null here depending on if
    //    this is the first time this method has been called;
    //    if first time then it's null and next call will
    //    have valid ServiceID
    ServiceItem item = new ServiceItem(serviceID, obj, null);
    ServiceRegistration reg = null;
    try {
      // use the first registrar to register
      reg = registrar.register(item, LEASE_DURATION);
      serviceRegistrations.add(reg);
      leaseMan.renewFor(reg.getLease(), LEASE_DURATION, LEASE_DURATION, null);
    }
    catch (RemoteException e) {
      if (DEBUG) e.printStackTrace();
      throw new JiniUtilException("problem registering object");
    }
    // save the service ID
    serviceID = reg.getServiceID();
  } // registerInOne

  /**
   * Register in all lookup services within multicast radius.  To prevent
   * multiple registrations of the same object in the same lookup service,
   * this method unregisters the object in all current lookup services then
   * registers again.
   * @throws JiniUtilException
   */
  public void registerInAll() throws JiniUtilException {
    unregisterInAll();
    // call registerInOne in order to get service ID
    DiscoveryUtil d = new DiscoveryUtil();
    Vector serviceRegistrars = d.discoverAll();
    ServiceItem item = new ServiceItem(serviceID, obj, null);
    ServiceRegistration reg = null;
    for (int i = 0; i < serviceRegistrars.size(); i++) {
      try {
        reg = ((ServiceRegistrar) serviceRegistrars.elementAt(i)).register(item, LEASE_DURATION);
        // save the service ID
        serviceID = reg.getServiceID();
        serviceRegistrations.add(reg);
        leaseMan.renewFor(reg.getLease(), LEASE_DURATION, LEASE_DURATION, null);
      }
      catch (RemoteException e) {
        if (DEBUG) e.printStackTrace();
        // don't signal anything wrong if a particular registration fails
      }
    }
  } // registerInAll

  /**
   * Cancels leases in all lookup services for which this object is registered.
   */
  public void unregisterInAll() {
    for (int i = 0; i < serviceRegistrations.size(); i++) {
      try {
        leaseMan.cancel(((ServiceRegistration)serviceRegistrations.remove(0)).getLease());
      }
      catch (RemoteException e) {
        if (DEBUG) e.printStackTrace();
      }
      catch (UnknownLeaseException e) {
        if (DEBUG) e.printStackTrace();
      }
    }
  } // unregisterInAll

  public static void main(String[] args) throws Exception {
    System.setSecurityManager(new RMISecurityManager());
    ServiceRegistrationUtil regUtil = new ServiceRegistrationUtil(new String("Hello world!"));
    System.out.println("Registering object...");
    // regUtil.registerInAll();
    regUtil.register("localhost");
    System.out.println("Sleeping...");
    Thread.currentThread().sleep(1000*10);
    // you can simulate a crash by not unregistering....all leases should expire
    System.out.println("Unregistering object...");
    regUtil.unregisterInAll();
  } // main

  /**
   * Returns the object which this util has registered.
   * @return the object
   */
  public Object getObject() {
    return obj;
  }

} // ServiceRegistrationUtil