package edu.ucf.cs.dml.simnets.measureme.server;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: </p>
 * @author Mathew B. Lowery
 * @version 1.0
 */
import net.jini.core.lookup.ServiceTemplate;
import java.util.Vector;
import net.jini.core.lookup.ServiceRegistrar;
import java.rmi.RemoteException;
import net.jini.space.JavaSpace;
import java.rmi.RMISecurityManager;

/**
 * A convenience class for finding service objects.
 * @see JiniUtilException
 * @see DiscoveryUtil
 * @see ServiceRegistrationUtil
 */
public class ServiceSearchUtil {
  private static final boolean DEBUG = true;

  public ServiceSearchUtil() {
  } // constructor

  /**
   * Returns object that implements all interfaces in <code>classes</code>.  This
   * object could be registered in any Jini lookup service within the multicast
   * radius of this node.  <code>null</code> is returned if there are no matches
   * or no lookup services could be located.
   */
  public Object findOneMatch(Class[] classes) throws JiniUtilException {
    return findOneMatch(new ServiceTemplate(null, classes, null));
  } // findOneMatch

  /**
   * Same as <code>findOneMatch(Class[] classes)</code> but allows for better
   * matching by using a <code>ServiceTemplate</code> object.
   */
  public Object findOneMatch(ServiceTemplate template) throws JiniUtilException {
    DiscoveryUtil d = new DiscoveryUtil();
    Vector registrars = d.discoverAll();
    if (registrars.isEmpty()) {
      throw new JiniUtilException("no lookup services available");
    }
    Object serviceObject = null;
    for (int i = 0; i < registrars.size(); i++) {
      try {
        serviceObject = ((ServiceRegistrar) registrars.elementAt(i)).lookup(template);
        if (serviceObject != null) {
          return serviceObject;
        }
      }
      catch (RemoteException e) {
        if (DEBUG) e.printStackTrace();
        throw new JiniUtilException("problem during lookup");
      }
    }
    return null;
  } // findOneMatch

  /**
   * Test the class here.  In order to run this class, the parameter
   * <code>-Djava.security.policy=<path to policy file></code> must exist.
   * This is because an instance of <code>JavaSpace</code> is serialized
   * to this node and the actual implementation class is downloaded.  Permission
   * must be given for this class to be run.
   */
  public static void main(String[] args) throws JiniUtilException {
    System.setSecurityManager(new RMISecurityManager());
    System.out.println("Searching for instance of JavaSpace...");
    ServiceSearchUtil searchUtil = new ServiceSearchUtil();
    Class[] classes = {JavaSpace.class};
    JavaSpace space = (JavaSpace) searchUtil.findOneMatch(classes);
    if (space != null) {
      System.out.println("Got instance of JavaSpace: " + space.toString());
    }
    else {
      System.out.println("No matching JavaSpaces.");
    }
  } // main
} // ServiceSearchUtil